@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2015 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -22,8 +22,8 @@
 
 #include "config.h"
 
-#include <nfft3.h>
-#include <nfft3util.h>
+#include "nfft3.h"
+#include "infft.h"
 
 #define NREPEAT 5
 
@@ -31,7 +31,7 @@ static FILE* file_out_tex = NULL;
 
 int get_nthreads_array(int **arr)
 {
-  int max_threads = nfft_get_omp_num_threads();
+  int max_threads = NFFT(get_num_threads)();
   int alloc_num = 2;
   int k;
   int ret_number = 0;
@@ -39,28 +39,29 @@ int get_nthreads_array(int **arr)
 
   if (max_threads <= 5)
   {
-    *arr = (int*) malloc(max_threads*sizeof(int));
+    *arr = (int*) NFFT(malloc)((size_t) (max_threads) * sizeof(int));
     for (k = 0; k < max_threads; k++)
-      *(*arr + k) = k+1;
+      *(*arr + k) = k + 1;
     return max_threads;
   }
 
-  for (k = 1; k <= max_threads; k*=2, alloc_num++);
+  for (k = 1; k <= max_threads; k *= 2, alloc_num++)
+    ;
 
-  *arr = (int*) malloc(alloc_num*sizeof(int));
+  *arr = (int*) NFFT(malloc)((size_t)(alloc_num) * sizeof(int));
 
-  for (k = 1; k <= max_threads; k*=2)
+  for (k = 1; k <= max_threads; k *= 2)
   {
-    if (k != max_threads && 2*k > max_threads && max_threads_pw2)
+    if (k != max_threads && 2 * k > max_threads && max_threads_pw2)
     {
-      *(*arr + ret_number) = max_threads/2;
+      *(*arr + ret_number) = max_threads / 2;
       ret_number++;
     }
 
     *(*arr + ret_number) = k;
     ret_number++;
 
-    if (k != max_threads && 2*k > max_threads)
+    if (k != max_threads && 2 * k > max_threads)
     {
       *(*arr + ret_number) = max_threads;
       ret_number++;
@@ -69,8 +70,7 @@ int get_nthreads_array(int **arr)
   }
 
   return ret_number;
-} 
-  
+}
 
 void check_result_value(const int val, const int ok, const char *msg)
 {
@@ -78,7 +78,7 @@ void check_result_value(const int val, const int ok, const char *msg)
   {
     fprintf(stderr, "ERROR %s: %d not %d\n", msg, val, ok);
 
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 }
 
@@ -86,7 +86,9 @@ void run_test_create(int d, int L, int M)
 {
   char cmd[1025];
 
-  snprintf(cmd, 1024, "./fastsum_benchomp_createdataset %d %d %d > fastsum_benchomp_test.data", d, L, M);
+  snprintf(cmd, 1024,
+      "./fastsum_benchomp_createdataset %d %d %d > fastsum_benchomp_test.data",
+      d, L, M);
   fprintf(stderr, "%s\n", cmd);
   check_result_value(system(cmd), 0, "createdataset");
 }
@@ -94,7 +96,7 @@ void run_test_create(int d, int L, int M)
 void run_test_init_output()
 {
   FILE *f = fopen("fastsum_benchomp_test.result", "w");
-  if (f!= NULL)
+  if (f != NULL)
     fclose(f);
 }
 
@@ -107,16 +109,16 @@ typedef struct
   int m;
   int p;
   char *kernel_name;
-  double c;
-  double eps_I;
-  double eps_B;
+  R c;
+  R eps_I;
+  R eps_B;
 } s_param;
 
 typedef struct
 {
-  double avg;
-  double min;
-  double max;
+  R avg;
+  R min;
+  R max;
 } s_resval;
 
 typedef struct
@@ -132,31 +134,41 @@ typedef struct
   int nresults;
 } s_testset;
 
-void run_test(s_resval *res, int nrepeat, int n, int m, int p, char *kernel_name, double c, double eps_I, double eps_B, int nthreads)
+void run_test(s_resval *res, int nrepeat, int n, int m, int p,
+    char *kernel_name, R c, R eps_I, R eps_B, int nthreads)
 {
   char cmd[1025];
-  int r,t;
-  
+  int r, t;
+
   for (t = 0; t < 16; t++)
   {
-    res[t].avg = 0.0; res[t].min = 1.0/0.0; res[t].max = 0.0;
+    res[t].avg = K(0.0);
+    res[t].min = K(1.0) / K(0.0);
+    res[t].max = K(0.0);
   }
 
   if (nthreads < 2)
-    snprintf(cmd, 1024, "./fastsum_benchomp_detail_single %d %d %d %s %lg %lg %lg < fastsum_benchomp_test.data > fastsum_benchomp_test.out", n, m, p, kernel_name, c, eps_I, eps_B);
+    snprintf(cmd, 1024,
+        "./fastsum_benchomp_detail_single %d %d %d %s " __FR__ " " __FR__ " " __FR__ " < fastsum_benchomp_test.data > fastsum_benchomp_test.out",
+        n, m, p, kernel_name, c, eps_I, eps_B);
   else
-    snprintf(cmd, 1024, "./fastsum_benchomp_detail_threads %d %d %d %s %lg %lg %lg %d < fastsum_benchomp_test.data > fastsum_benchomp_test.out", n, m, p, kernel_name, c, eps_I, eps_B, nthreads);
+    snprintf(cmd, 1024,
+        "./fastsum_benchomp_detail_threads %d %d %d %s " __FR__ " " __FR__ " " __FR__ " %d < fastsum_benchomp_test.data > fastsum_benchomp_test.out",
+        n, m, p, kernel_name, c, eps_I, eps_B, nthreads);
   fprintf(stderr, "%s\n", cmd);
   check_result_value(system(cmd), 0, cmd);
 
   for (r = 0; r < nrepeat; r++)
   {
     int retval;
-    double v[16];
+    R v[16];
     FILE *f;
     check_result_value(system(cmd), 0, cmd);
     f = fopen("fastsum_benchomp_test.out", "r");
-    retval = fscanf(f, "%lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg", v, v+1, v+2, v+3, v+4, v+5, v+6, v+7, v+8, v+9, v+10, v+11, v+12, v+13, v+14, v+15);
+    retval = fscanf(f,
+        "" __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ " " __FR__ "", v,
+        v + 1, v + 2, v + 3, v + 4, v + 5, v + 6, v + 7, v + 8, v + 9, v + 10,
+        v + 11, v + 12, v + 13, v + 14, v + 15);
     check_result_value(retval, 16, "read fastsum_benchomp_test.out");
     fclose(f);
 
@@ -171,11 +183,11 @@ void run_test(s_resval *res, int nrepeat, int n, int m, int p, char *kernel_name
   }
 
   for (t = 0; t < 16; t++)
-    res[t].avg /= nrepeat;
+    res[t].avg /= (R)(nrepeat);
 
   fprintf(stderr, "%d %d: ", nthreads, nrepeat);
   for (t = 0; t < 16; t++)
-    fprintf(stderr, "%.3e %.3e %.3e | ", res[t].avg, res[t].min, res[t].max);
+    fprintf(stderr, "%.3" __FES__ " %.3" __FES__ " %.3" __FES__ " | ", res[t].avg, res[t].min, res[t].max);
   fprintf(stderr, "\n");
 }
 
@@ -204,7 +216,7 @@ const char *get_adjoint_omp_string(int flags)
   if (flags & NFFT_OMP_BLOCKWISE_ADJOINT)
     return "blockwise";
 
-    return "";
+  return "";
 }
 
 #define MASK_FSUM_D (1U<<0)
@@ -217,7 +229,8 @@ const char *get_adjoint_omp_string(int flags)
 #define MASK_FSUM_EPSI (1U<<7)
 #define MASK_FSUM_EPSB (1U<<8)
 
-unsigned int fastsum_determine_different_parameters(s_testset *testsets, int ntestsets)
+unsigned int fastsum_determine_different_parameters(s_testset *testsets,
+    int ntestsets)
 {
   int t;
   unsigned int mask = 0;
@@ -227,23 +240,24 @@ unsigned int fastsum_determine_different_parameters(s_testset *testsets, int nte
 
   for (t = 1; t < ntestsets; t++)
   {
-    if (testsets[t-1].param.d != testsets[t].param.d)
+    if (testsets[t - 1].param.d != testsets[t].param.d)
       mask |= MASK_FSUM_D;
-    if (testsets[t-1].param.L != testsets[t].param.L)
+    if (testsets[t - 1].param.L != testsets[t].param.L)
       mask |= MASK_FSUM_L;
-    if (testsets[t-1].param.M != testsets[t].param.M)
+    if (testsets[t - 1].param.M != testsets[t].param.M)
       mask |= MASK_FSUM_M;
-    if (testsets[t-1].param.n != testsets[t].param.n)
+    if (testsets[t - 1].param.n != testsets[t].param.n)
       mask |= MASK_FSUM_MULTIBW;
-    if (testsets[t-1].param.m != testsets[t].param.m)
+    if (testsets[t - 1].param.m != testsets[t].param.m)
       mask |= MASK_FSUM_WINM;
-    if (testsets[t-1].param.p != testsets[t].param.p)
+    if (testsets[t - 1].param.p != testsets[t].param.p)
       mask |= MASK_FSUM_P;
-    if (strcmp(testsets[t-1].param.kernel_name, testsets[t].param.kernel_name) != 0)
+    if (strcmp(testsets[t - 1].param.kernel_name, testsets[t].param.kernel_name)
+        != 0)
       mask |= MASK_FSUM_KERNEL;
-    if (testsets[t-1].param.eps_I != testsets[t].param.eps_I)
+    if (testsets[t - 1].param.eps_I != testsets[t].param.eps_I)
       mask |= MASK_FSUM_EPSI;
-    if (testsets[t-1].param.eps_B != testsets[t].param.eps_B)
+    if (testsets[t - 1].param.eps_B != testsets[t].param.eps_B)
       mask |= MASK_FSUM_EPSB;
   }
 
@@ -259,72 +273,81 @@ void strEscapeUnderscore(char *dst, char *src, int maxlen)
   while (src[i] != '\0' && len + offset < maxlen - 1)
   {
     if (src[i] == '_')
-      len = snprintf(dst+offset, maxlen-offset, "\\_{}");
+      len = snprintf(dst + offset, maxlen - offset, "\\_{}");
     else
-      len = snprintf(dst+offset, maxlen-offset, "%c", src[i]);
+      len = snprintf(dst + offset, maxlen - offset, "%c", src[i]);
     offset += len;
     i++;
   }
 }
 
-void fastsum_get_plot_title_minus_indep(char *outstr, int maxlen, char *hostname, s_param param, unsigned int diff_mask)
+void fastsum_get_plot_title_minus_indep(char *outstr, int maxlen,
+    char *hostname, s_param param, unsigned int diff_mask)
 {
   unsigned int mask = ~diff_mask;
   int offset = 0;
   int len;
 
   len = snprintf(outstr, maxlen, "%s", hostname);
-  if (len < 0 || len+offset >= maxlen-1) return;
+  if (len < 0 || len + offset >= maxlen - 1)
+    return;
   offset += len;
 
   if (mask & MASK_FSUM_D)
   {
-    len = snprintf(outstr+offset, maxlen-offset, " %dd fastsum", param.d);
-    if (len < 0 || len+offset >= maxlen-1) return;
+    len = snprintf(outstr + offset, maxlen - offset, " %dd fastsum", param.d);
+    if (len < 0 || len + offset >= maxlen - 1)
+      return;
     offset += len;
   }
 
   if ((mask & (MASK_FSUM_L | MASK_FSUM_M)) && param.L == param.M)
   {
-    len = snprintf(outstr+offset, maxlen-offset, " L=M=%d", param.L);
-    if (len < 0 || len+offset >= maxlen-1) return;
+    len = snprintf(outstr + offset, maxlen - offset, " L=M=%d", param.L);
+    if (len < 0 || len + offset >= maxlen - 1)
+      return;
     offset += len;
   }
   else
   {
     if (mask & MASK_FSUM_L)
     {
-      len = snprintf(outstr+offset, maxlen-offset, " L=%d", param.L);
-      if (len < 0 || len+offset >= maxlen-1) return;
+      len = snprintf(outstr + offset, maxlen - offset, " L=%d", param.L);
+      if (len < 0 || len + offset >= maxlen - 1)
+        return;
       offset += len;
     }
 
     if (mask & MASK_FSUM_M)
     {
-      len = snprintf(outstr+offset, maxlen-offset, " M=%d", param.M);
-      if (len < 0 || len+offset >= maxlen-1) return;
+      len = snprintf(outstr + offset, maxlen - offset, " M=%d", param.M);
+      if (len < 0 || len + offset >= maxlen - 1)
+        return;
       offset += len;
     }
   }
 
   if (mask & MASK_FSUM_MULTIBW)
   {
-    len = snprintf(outstr+offset, maxlen-offset, " n=%d", param.n);
-    if (len < 0 || len+offset >= maxlen-1) return;
+    len = snprintf(outstr + offset, maxlen - offset, " n=%d", param.n);
+    if (len < 0 || len + offset >= maxlen - 1)
+      return;
     offset += len;
   }
 
   if (mask & MASK_FSUM_WINM)
   {
-    len = snprintf(outstr+offset, maxlen-offset, " m=%d", param.m);
-    if (len < 0 || len+offset >= maxlen-1) return;
+    len = snprintf(outstr + offset, maxlen - offset, " m=%d", param.m);
+    if (len < 0 || len + offset >= maxlen - 1)
+      return;
     offset += len;
   }
 
   if (mask & MASK_FSUM_P)
   {
-    len = snprintf(outstr+offset, maxlen-offset, " p=%d", param.p);
-    if (len < 0 || len+offset >= maxlen-1) return;
+    len = snprintf(outstr + offset, maxlen - offset, " p=%d", param.p);
+    if (len < 0 || len + offset >= maxlen - 1)
+      return;
     offset += len;
   }
 
@@ -333,30 +356,38 @@ void fastsum_get_plot_title_minus_indep(char *outstr, int maxlen, char *hostname
     char tmp[maxlen];
     strEscapeUnderscore(tmp, param.kernel_name, maxlen);
 
-    len = snprintf(outstr+offset, maxlen-offset, " %s", tmp);
-    if (len < 0 || len+offset >= maxlen-1) return;
+    len = snprintf(outstr + offset, maxlen - offset, " %s", tmp);
+    if (len < 0 || len + offset >= maxlen - 1)
+      return;
     offset += len;
   }
 
   if ((mask & (MASK_FSUM_EPSI | MASK_FSUM_EPSB)) && param.eps_I == param.eps_B)
   {
-    len = snprintf(outstr+offset, maxlen-offset, " $\\varepsilon_\\mathrm{I}$=$\\varepsilon_\\mathrm{B}$=%g", param.eps_I);
-    if (len < 0 || len+offset >= maxlen-1) return;
+    len = snprintf(outstr + offset, maxlen - offset,
+        " $\\varepsilon_\\mathrm{I}$=$\\varepsilon_\\mathrm{B}$=%" __FGS__ "",
+        param.eps_I);
+    if (len < 0 || len + offset >= maxlen - 1)
+      return;
     offset += len;
   }
   else
   {
     if (mask & MASK_FSUM_EPSI)
     {
-      len = snprintf(outstr+offset, maxlen-offset, " $\\varepsilon_\\mathrm{I}$=%g", param.eps_I);
-      if (len < 0 || len+offset >= maxlen-1) return;
+      len = snprintf(outstr + offset, maxlen - offset,
+          " $\\varepsilon_\\mathrm{I}$=%" __FGS__ "", param.eps_I);
+      if (len < 0 || len + offset >= maxlen - 1)
+        return;
       offset += len;
     }
 
     if (mask & MASK_FSUM_EPSB)
     {
-      len = snprintf(outstr+offset, maxlen-offset, " $\\varepsilon_\\mathrm{B}$=%g", param.eps_B);
-      if (len < 0 || len+offset >= maxlen-1) return;
+      len = snprintf(outstr + offset, maxlen - offset,
+          " $\\varepsilon_\\mathrm{B}$=%" __FGS__ "", param.eps_B);
+      if (len < 0 || len + offset >= maxlen - 1)
+        return;
       offset += len;
     }
   }
@@ -380,34 +411,44 @@ void nfft_adjoint_print_output_histo_DFBRT(FILE *out, s_testset testset)
     else
       fprintf(out, "%d", testset.results[i].nthreads);
 
-  fprintf(out, "}, x tick label style={ /pgf/number format/1000 sep=}, xlabel=Number of threads, ylabel=Time in s, xtick=data, legend style={legend columns=-1}, ybar, bar width=7pt, ymajorgrids=true, yminorgrids=true, minor y tick num=1, ");
-  fprintf(out, " title={%s %dd $\\textrm{NFFT}^\\top$ N=%d $\\sigma$=2 M=%d m=%d prepsi sorted}", hostname, testset.param.d, testset.param.n, testset.param.M, testset.param.m);
+  fprintf(out,
+      "}, x tick label style={ /pgf/number format/1000 sep=}, xlabel=Number of threads, ylabel=Time in s, xtick=data, legend style={legend columns=-1}, ybar, bar width=7pt, ymajorgrids=true, yminorgrids=true, minor y tick num=1, ");
+  fprintf(out,
+      " title={%s %dd $\\textrm{NFFT}^\\top$ N=%d $\\sigma$=2 M=%d m=%d prepsi sorted}",
+      hostname, testset.param.d, testset.param.n, testset.param.M,
+      testset.param.m);
   fprintf(out, " ]\n");
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[10].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[10].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[11].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[11].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[12].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[12].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[1].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[1].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[4].avg + testset.results[i].resval[1].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[4].avg + testset.results[i].resval[1].avg);
   fprintf(out, "};\n");
-  fprintf(out, "\\legend{D,$\\textrm{F}^\\top$,$\\textrm{B}^\\top$,prepsi,total}\n");
+  fprintf(out,
+      "\\legend{D,$\\textrm{F}^\\top$,$\\textrm{B}^\\top$,prepsi,total}\n");
   fprintf(out, "\\end{axis}\n");
   fprintf(out, "\\end{tikzpicture}\n");
   fprintf(out, "\n\n");
@@ -433,32 +474,41 @@ void nfft_trafo_print_output_histo_DFBRT(FILE *out, s_testset testset)
     else
       fprintf(out, "%d", testset.results[i].nthreads);
 
-  fprintf(out, "}, x tick label style={ /pgf/number format/1000 sep=}, xlabel=Number of threads, ylabel=Time in s, xtick=data, legend style={legend columns=-1}, ybar, bar width=7pt, ymajorgrids=true, yminorgrids=true, minor y tick num=1, ");
-  fprintf(out, " title={%s %dd $\\textrm{NFFT}$ N=%d $\\sigma$=2 M=%d m=%d prepsi sorted}", hostname, testset.param.d, testset.param.n, testset.param.M, testset.param.m);
+  fprintf(out,
+      "}, x tick label style={ /pgf/number format/1000 sep=}, xlabel=Number of threads, ylabel=Time in s, xtick=data, legend style={legend columns=-1}, ybar, bar width=7pt, ymajorgrids=true, yminorgrids=true, minor y tick num=1, ");
+  fprintf(out,
+      " title={%s %dd $\\textrm{NFFT}$ N=%d $\\sigma$=2 M=%d m=%d prepsi sorted}",
+      hostname, testset.param.d, testset.param.n, testset.param.M,
+      testset.param.m);
   fprintf(out, " ]\n");
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[13].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[13].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[14].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[14].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[15].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[15].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[2].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[2].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[6].avg + testset.results[i].resval[2].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[6].avg + testset.results[i].resval[2].avg);
   fprintf(out, "};\n");
   fprintf(out, "\\legend{D,F,B,prepsi,total}\n");
   fprintf(out, "\\end{axis}\n");
@@ -477,7 +527,8 @@ void fastsum_print_output_histo_PreRfNfT(FILE *out, s_testset testset)
   if (gethostname(hostname, 1024) != 0)
     strncpy(hostname, "unnamed", 1024);
 
-  fastsum_get_plot_title_minus_indep(plottitle, 1024, hostname, testset.param, 0);
+  fastsum_get_plot_title_minus_indep(plottitle, 1024, hostname, testset.param,
+      0);
 
   fprintf(out, "\\begin{tikzpicture}\n");
   fprintf(out, "\\begin{axis}[");
@@ -489,34 +540,42 @@ void fastsum_print_output_histo_PreRfNfT(FILE *out, s_testset testset)
     else
       fprintf(out, "%d", testset.results[i].nthreads);
 
-  fprintf(out, "}, x tick label style={ /pgf/number format/1000 sep=}, xlabel=Number of threads, ylabel=Time in s, xtick=data, legend style={legend columns=1}, ybar, bar width=7pt, ymajorgrids=true, yminorgrids=true, minor y tick num=1, ");
+  fprintf(out,
+      "}, x tick label style={ /pgf/number format/1000 sep=}, xlabel=Number of threads, ylabel=Time in s, xtick=data, legend style={legend columns=1}, ybar, bar width=7pt, ymajorgrids=true, yminorgrids=true, minor y tick num=1, ");
   fprintf(out, " title={%s}", plottitle);
   fprintf(out, " ]\n");
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[1].avg+testset.results[i].resval[2].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[1].avg + testset.results[i].resval[2].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[3].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[3].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[4].avg + testset.results[i].resval[5].avg + testset.results[i].resval[6].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[4].avg + testset.results[i].resval[5].avg
+            + testset.results[i].resval[6].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[7].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[7].avg);
   fprintf(out, "};\n");
 
   fprintf(out, "\\addplot coordinates {");
   for (i = 0; i < size; i++)
-    fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, testset.results[i].resval[9].avg - testset.results[i].resval[0].avg);
+    fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+        testset.results[i].resval[9].avg - testset.results[i].resval[0].avg);
   fprintf(out, "};\n");
-  fprintf(out, "\\legend{prepsi (step 1b),init nearfield (step 1c),far field (steps 2a-c),nearfield (step 2d),total $-$ step 1a}\n");
+  fprintf(out,
+      "\\legend{prepsi (step 1b),init nearfield (step 1c),far field (steps 2a-c),nearfield (step 2d),total $-$ step 1a}\n");
   fprintf(out, "\\end{axis}\n");
   fprintf(out, "\\end{tikzpicture}\n");
   fprintf(out, "\n\n");
@@ -524,21 +583,25 @@ void fastsum_print_output_histo_PreRfNfT(FILE *out, s_testset testset)
   fflush(out);
 }
 
-void fastsum_print_output_speedup_total_minus_indep(FILE *out, s_testset *testsets, int ntestsets)
+void fastsum_print_output_speedup_total_minus_indep(FILE *out,
+    s_testset *testsets, int ntestsets)
 {
   int i, t;
   char hostname[1025];
   char plottitle[1025];
-  unsigned int diff_mask = fastsum_determine_different_parameters(testsets, ntestsets);
+  unsigned int diff_mask = fastsum_determine_different_parameters(testsets,
+      ntestsets);
 
   if (gethostname(hostname, 1024) != 0)
     strncpy(hostname, "unnamed", 1024);
 
-  fastsum_get_plot_title_minus_indep(plottitle, 1024, hostname, testsets[0].param, diff_mask | MASK_FSUM_WINM);
+  fastsum_get_plot_title_minus_indep(plottitle, 1024, hostname,
+      testsets[0].param, diff_mask | MASK_FSUM_WINM);
 
   fprintf(out, "\\begin{tikzpicture}\n");
   fprintf(out, "\\begin{axis}[");
-  fprintf(out, "width=0.9\\textwidth, height=0.6\\textwidth, x tick label style={ /pgf/number format/1000 sep=}, xlabel=Number of threads, ylabel=Speedup, xtick=data, legend style={ legend pos = north west, legend columns=1}, ymajorgrids=true, yminorgrids=true, minor y tick num=4, ");
+  fprintf(out,
+      "width=0.9\\textwidth, height=0.6\\textwidth, x tick label style={ /pgf/number format/1000 sep=}, xlabel=Number of threads, ylabel=Speedup, xtick=data, legend style={ legend pos = north west, legend columns=1}, ymajorgrids=true, yminorgrids=true, minor y tick num=4, ");
   fprintf(out, " title={%s}", plottitle);
   fprintf(out, " ]\n");
 
@@ -546,19 +609,26 @@ void fastsum_print_output_speedup_total_minus_indep(FILE *out, s_testset *testse
   {
     s_testset testset = testsets[t];
 
-    double tref = 0.0;
+    R tref = K(0.0);
     for (i = 0; i < testset.nresults; i++)
       if (testset.results[i].nthreads == 1)
-        tref = testset.results[i].resval[9].avg - testset.results[i].resval[0].avg;
+        tref = testset.results[i].resval[9].avg
+            - testset.results[i].resval[0].avg;
 
     fprintf(out, "\\addplot coordinates {");
     for (i = 0; i < testset.nresults; i++)
-      fprintf(out, "(%d, %.6e) ", testset.results[i].nthreads, tref/(testset.results[i].resval[9].avg - testset.results[i].resval[0].avg));
+      fprintf(out, "(%d, %.6" __FES__ ") ", testset.results[i].nthreads,
+          tref
+              / (testset.results[i].resval[9].avg
+                  - testset.results[i].resval[0].avg));
     fprintf(out, "};\n");
 
     for (i = 0; i < testset.nresults; i++)
     {
-      fprintf(stderr, "%d:%.3f  ", testset.results[i].nthreads, tref/(testset.results[i].resval[9].avg - testset.results[i].resval[0].avg));
+      fprintf(stderr, "%d:%.3" __FIS__ "  ", testset.results[i].nthreads,
+          tref
+              / (testset.results[i].resval[9].avg
+                  - testset.results[i].resval[0].avg));
     }
     fprintf(stderr, "\n\n");
   }
@@ -569,7 +639,8 @@ void fastsum_print_output_speedup_total_minus_indep(FILE *out, s_testset *testse
     char title[256];
     if (t > 0)
       fprintf(out, "},{");
-    fastsum_get_plot_title_minus_indep(title, 255, "", testsets[t].param, ~(diff_mask | MASK_FSUM_WINM));
+    fastsum_get_plot_title_minus_indep(title, 255, "", testsets[t].param,
+        ~(diff_mask | MASK_FSUM_WINM));
     fprintf(out, "%s", title);
   }
   fprintf(out, "}}\n");
@@ -580,7 +651,9 @@ void fastsum_print_output_speedup_total_minus_indep(FILE *out, s_testset *testse
   fflush(out);
 }
 
-void run_testset(s_testset *testset, int d, int L, int M, int n, int m, int p, char *kernel_name, double c, double eps_I, double eps_B, int *nthreads_array, int n_threads_array_size)
+void run_testset(s_testset *testset, int d, int L, int M, int n, int m, int p,
+    char *kernel_name, R c, R eps_I, R eps_B,
+    int *nthreads_array, int n_threads_array_size)
 {
   int i;
   testset->param.d = d;
@@ -594,14 +667,18 @@ void run_testset(s_testset *testset, int d, int L, int M, int n, int m, int p, c
   testset->param.eps_I = eps_I;
   testset->param.eps_B = eps_B;
 
-  testset->results = (s_result*) malloc(n_threads_array_size*sizeof(s_result));
+  testset->results = (s_result*) NFFT(malloc)(
+      (size_t)(n_threads_array_size) * sizeof(s_result));
   testset->nresults = n_threads_array_size;
 
   run_test_create(testset->param.d, testset->param.L, testset->param.M);
   for (i = 0; i < n_threads_array_size; i++)
   {
     testset->results[i].nthreads = nthreads_array[i];
-    run_test(testset->results[i].resval, NREPEAT, testset->param.n, testset->param.m, testset->param.p, testset->param.kernel_name, testset->param.c, testset->param.eps_I, testset->param.eps_B, testset->results[i].nthreads);
+    run_test(testset->results[i].resval, NREPEAT, testset->param.n,
+        testset->param.m, testset->param.p, testset->param.kernel_name,
+        testset->param.c, testset->param.eps_I, testset->param.eps_B,
+        testset->results[i].nthreads);
   }
 
 }
@@ -611,7 +688,7 @@ void test1(int *nthreads_array, int n_threads_array_size)
   s_testset testsets[1];
 
 #if defined MEASURE_TIME && defined MEASURE_TIME_FFTW
-  run_testset(&testsets[0], 3, 100000, 100000, 128, 4, 7, "one_over_x", 0.0, 0.03125, 0.03125, nthreads_array, n_threads_array_size);
+  run_testset(&testsets[0], 3, 100000, 100000, 128, 4, 7, "one_over_x", K(0.0), K(0.03125), K(0.03125), nthreads_array, n_threads_array_size);
 
   fastsum_print_output_speedup_total_minus_indep(file_out_tex, testsets, 1);
 
@@ -632,7 +709,8 @@ int main(int argc, char** argv)
 #if !(defined MEASURE_TIME && defined MEASURE_TIME_FFTW)
   fprintf(stderr, "WARNING: Detailed time measurements are not activated.\n");
   fprintf(stderr, "Please re-run the configure script with options\n");
-  fprintf(stderr, "--enable-measure-time --enable-measure-time-fftw --enable-openmp\n");
+  fprintf(stderr,
+      "--enable-measure-time --enable-measure-time-fftw --enable-openmp\n");
   fprintf(stderr, "and run \"make clean all\"\n\n");
 #endif
 
@@ -646,6 +724,6 @@ int main(int argc, char** argv)
 
   fclose(file_out_tex);
 
-  return 0;
+  return EXIT_SUCCESS;
 }
 
