@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2015 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -16,7 +16,7 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: quadratureS2.c 3896 2012-10-10 12:19:26Z tovo $ */
+/* $Id$ */
 
 /**
  * \defgroup applications_quadratureS2_test quadratureS2_test
@@ -36,7 +36,6 @@
 #endif
 
 /* Include NFFT 3 utilities headers. */
-#include "nfft3util.h"
 
 /* Include NFFT 3 library header. */
 #include "nfft3.h"
@@ -246,7 +245,7 @@ int main (int argc, char **argv)
           if (temp <= 1)
           {
             x_compare[2*d+1] = acos(x3);
-            if (x_compare[2*d+1] == 0 || x_compare[2*d+1] == PI)
+            if (x_compare[2*d+1] == 0 || x_compare[2*d+1] == KPI)
             {
               x_compare[2*d] = 0.0;
             }
@@ -254,8 +253,8 @@ int main (int argc, char **argv)
             {
               x_compare[2*d] = atan2(x2/sin(x_compare[2*d+1]),x1/sin(x_compare[2*d+1]));
             }
-            x_compare[2*d] *= 1.0/(2.0*PI);
-            x_compare[2*d+1] *= 1.0/(2.0*PI);
+            x_compare[2*d] *= 1.0/(2.0*KPI);
+            x_compare[2*d+1] *= 1.0/(2.0*KPI);
             d++;
           }
         }
@@ -288,16 +287,16 @@ int main (int argc, char **argv)
         /* Read cut-off degree and grid size parameter. */
         fscanf(stdin,"%d %d %d\n",&NQ[iNQ],&SQ[iNQ],&RQ[iNQ]);
         fprintf(stdout,"%d %d %d\n",NQ[iNQ],SQ[iNQ],RQ[iNQ]);
-        NQ_max = NFFT_MAX(NQ_max,NQ[iNQ]);
-        SQ_max = NFFT_MAX(SQ_max,SQ[iNQ]);
+        NQ_max = MAX(NQ_max,NQ[iNQ]);
+        SQ_max = MAX(SQ_max,SQ[iNQ]);
       }
       else
       {
         /* Read cut-off degree and grid size parameter. */
         fscanf(stdin,"%d %d\n",&NQ[iNQ],&SQ[iNQ]);
         fprintf(stdout,"%d %d\n",NQ[iNQ],SQ[iNQ]);
-        NQ_max = NFFT_MAX(NQ_max,NQ[iNQ]);
-        SQ_max = NFFT_MAX(SQ_max,SQ[iNQ]);
+        NQ_max = MAX(NQ_max,NQ[iNQ]);
+        SQ_max = MAX(SQ_max,SQ[iNQ]);
       }
     }
 
@@ -397,9 +396,9 @@ int main (int argc, char **argv)
             //fprintf(stderr,"ed: m_theta = %d\n",m_theta);
             for (k = 1; k < SQ[iNQ]; k++)
             {
-              m_theta += (int)floor((2*PI)/acos((cos(PI/(double)SQ[iNQ])-
-                cos(k*PI/(double)SQ[iNQ])*cos(k*PI/(double)SQ[iNQ]))/
-                (sin(k*PI/(double)SQ[iNQ])*sin(k*PI/(double)SQ[iNQ]))));
+              m_theta += (int)floor((2*KPI)/acos((cos(KPI/(double)SQ[iNQ])-
+                cos(k*KPI/(double)SQ[iNQ])*cos(k*KPI/(double)SQ[iNQ]))/
+                (sin(k*KPI/(double)SQ[iNQ])*sin(k*KPI/(double)SQ[iNQ]))));
               //fprintf(stderr,"ed: m_theta = %d\n",m_theta);
             }
             //fprintf(stderr,"ed: m_theta final = %d\n",m_theta);
@@ -424,7 +423,7 @@ int main (int argc, char **argv)
             for (k = 0; k < m_theta; k++)
             {
               fscanf(stdin,"%le\n",&w[k]);
-              w[k] *= (2.0*PI)/((double)m_phi);
+              w[k] *= (2.0*KPI)/((double)m_phi);
             }
 
             //fprintf(stderr,"Allocating theta and phi\n");
@@ -506,7 +505,7 @@ int main (int argc, char **argv)
 
             for (k = 0; k < SQ[iNQ]+1; k++)
             {
-              w[k] *= (2.0*PI)/((double)(m_theta-1)*m_phi);
+              w[k] *= (2.0*KPI)/((double)(m_theta-1)*m_phi);
               w[m_theta-1-k] = w[k];
             }
             fftw_destroy_plan(fplan);
@@ -569,10 +568,10 @@ int main (int argc, char **argv)
 
             for (d = 0; d < m_total; d++)
             {
-              x_grid[2*d+1] = acos(x_grid[2*d+1])/(2.0*PI);
+              x_grid[2*d+1] = acos(x_grid[2*d+1])/(2.0*KPI);
             }
 
-            w[0] = (4.0*PI)/(m_total);
+            w[0] = (4.0*KPI)/(m_total);
             break;
 
           case GRID_EQUIDISTRIBUTION:
@@ -592,7 +591,7 @@ int main (int argc, char **argv)
 
               for (k = 0; k < SQ[iNQ]/2+1; k++)
               {
-                w_temp[k] *= (2.0*PI)/((double)(SQ[iNQ]));
+                w_temp[k] *= (2.0*KPI)/((double)(SQ[iNQ]));
                 w_temp[SQ[iNQ]-k] = w_temp[k];
               }
               fftw_destroy_plan(fplan);
@@ -607,7 +606,7 @@ int main (int argc, char **argv)
             }
             else
             {
-              w[d] = (4.0*PI)/(m_total);
+              w[d] = (4.0*KPI)/(m_total);
             }
             d = 1;
             x_grid[2*d] = -0.5;
@@ -618,28 +617,28 @@ int main (int argc, char **argv)
             }
             else
             {
-              w[d] = (4.0*PI)/(m_total);
+              w[d] = (4.0*KPI)/(m_total);
             }
             d = 2;
 
             for (k = 1; k < SQ[iNQ]; k++)
             {
-              theta_s = (double)k*PI/(double)SQ[iNQ];
-              M = (int)floor((2.0*PI)/acos((cos(PI/(double)SQ[iNQ])-
+              theta_s = (double)k*KPI/(double)SQ[iNQ];
+              M = (int)floor((2.0*KPI)/acos((cos(KPI/(double)SQ[iNQ])-
                 cos(theta_s)*cos(theta_s))/(sin(theta_s)*sin(theta_s))));
 
               for (n = 0; n < M; n++)
               {
                 x_grid[2*d] = (n + 0.5)/M;
                 x_grid[2*d] -= (x_grid[2*d]>=0.5)?(1.0):(0.0);
-                x_grid[2*d+1] = theta_s/(2.0*PI);
+                x_grid[2*d+1] = theta_s/(2.0*KPI);
                 if (gridtype == GRID_EQUIDISTRIBUTION)
                 {
                   w[d] = w_temp[k]/((double)(M));
                 }
                 else
                 {
-                  w[d] = (4.0*PI)/(m_total);
+                  w[d] = (4.0*KPI)/(m_total);
                 }
                 d++;
               }
@@ -757,18 +756,18 @@ int main (int argc, char **argv)
           case FUNCTION_F1:
             for (d = 0; d < m_total; d++)
             {
-              x1 = sin(x_grid[2*d+1]*2.0*PI)*cos(x_grid[2*d]*2.0*PI);
-              x2 = sin(x_grid[2*d+1]*2.0*PI)*sin(x_grid[2*d]*2.0*PI);
-              x3 = cos(x_grid[2*d+1]*2.0*PI);
+              x1 = sin(x_grid[2*d+1]*2.0*KPI)*cos(x_grid[2*d]*2.0*KPI);
+              x2 = sin(x_grid[2*d+1]*2.0*KPI)*sin(x_grid[2*d]*2.0*KPI);
+              x3 = cos(x_grid[2*d+1]*2.0*KPI);
               f_grid[d] = x1*x2*x3;
             }
             if (mode == RANDOM)
             {
               for (d = 0; d < m_compare; d++)
               {
-                x1 = sin(x_compare[2*d+1]*2.0*PI)*cos(x_compare[2*d]*2.0*PI);
-                x2 = sin(x_compare[2*d+1]*2.0*PI)*sin(x_compare[2*d]*2.0*PI);
-                x3 = cos(x_compare[2*d+1]*2.0*PI);
+                x1 = sin(x_compare[2*d+1]*2.0*KPI)*cos(x_compare[2*d]*2.0*KPI);
+                x2 = sin(x_compare[2*d+1]*2.0*KPI)*sin(x_compare[2*d]*2.0*KPI);
+                x3 = cos(x_compare[2*d+1]*2.0*KPI);
                 f_compare[d] = x1*x2*x3;
               }
             }
@@ -780,18 +779,18 @@ int main (int argc, char **argv)
           case FUNCTION_F2:
             for (d = 0; d < m_total; d++)
             {
-              x1 = sin(x_grid[2*d+1]*2.0*PI)*cos(x_grid[2*d]*2.0*PI);
-              x2 = sin(x_grid[2*d+1]*2.0*PI)*sin(x_grid[2*d]*2.0*PI);
-              x3 = cos(x_grid[2*d+1]*2.0*PI);
+              x1 = sin(x_grid[2*d+1]*2.0*KPI)*cos(x_grid[2*d]*2.0*KPI);
+              x2 = sin(x_grid[2*d+1]*2.0*KPI)*sin(x_grid[2*d]*2.0*KPI);
+              x3 = cos(x_grid[2*d+1]*2.0*KPI);
               f_grid[d] = 0.1*exp(x1+x2+x3);
             }
             if (mode == RANDOM)
             {
               for (d = 0; d < m_compare; d++)
               {
-                x1 = sin(x_compare[2*d+1]*2.0*PI)*cos(x_compare[2*d]*2.0*PI);
-                x2 = sin(x_compare[2*d+1]*2.0*PI)*sin(x_compare[2*d]*2.0*PI);
-                x3 = cos(x_compare[2*d+1]*2.0*PI);
+                x1 = sin(x_compare[2*d+1]*2.0*KPI)*cos(x_compare[2*d]*2.0*KPI);
+                x2 = sin(x_compare[2*d+1]*2.0*KPI)*sin(x_compare[2*d]*2.0*KPI);
+                x3 = cos(x_compare[2*d+1]*2.0*KPI);
                 f_compare[d] = 0.1*exp(x1+x2+x3);
               }
             }
@@ -803,9 +802,9 @@ int main (int argc, char **argv)
           case FUNCTION_F3:
             for (d = 0; d < m_total; d++)
             {
-              x1 = sin(x_grid[2*d+1]*2.0*PI)*cos(x_grid[2*d]*2.0*PI);
-              x2 = sin(x_grid[2*d+1]*2.0*PI)*sin(x_grid[2*d]*2.0*PI);
-              x3 = cos(x_grid[2*d+1]*2.0*PI);
+              x1 = sin(x_grid[2*d+1]*2.0*KPI)*cos(x_grid[2*d]*2.0*KPI);
+              x2 = sin(x_grid[2*d+1]*2.0*KPI)*sin(x_grid[2*d]*2.0*KPI);
+              x3 = cos(x_grid[2*d+1]*2.0*KPI);
               temp = sqrt(x1*x1)+sqrt(x2*x2)+sqrt(x3*x3);
               f_grid[d] = 0.1*temp;
             }
@@ -813,9 +812,9 @@ int main (int argc, char **argv)
             {
               for (d = 0; d < m_compare; d++)
               {
-                x1 = sin(x_compare[2*d+1]*2.0*PI)*cos(x_compare[2*d]*2.0*PI);
-                x2 = sin(x_compare[2*d+1]*2.0*PI)*sin(x_compare[2*d]*2.0*PI);
-                x3 = cos(x_compare[2*d+1]*2.0*PI);
+                x1 = sin(x_compare[2*d+1]*2.0*KPI)*cos(x_compare[2*d]*2.0*KPI);
+                x2 = sin(x_compare[2*d+1]*2.0*KPI)*sin(x_compare[2*d]*2.0*KPI);
+                x3 = cos(x_compare[2*d+1]*2.0*KPI);
                 temp = sqrt(x1*x1)+sqrt(x2*x2)+sqrt(x3*x3);
                 f_compare[d] = 0.1*temp;
               }
@@ -828,9 +827,9 @@ int main (int argc, char **argv)
           case FUNCTION_F4:
             for (d = 0; d < m_total; d++)
             {
-              x1 = sin(x_grid[2*d+1]*2.0*PI)*cos(x_grid[2*d]*2.0*PI);
-              x2 = sin(x_grid[2*d+1]*2.0*PI)*sin(x_grid[2*d]*2.0*PI);
-              x3 = cos(x_grid[2*d+1]*2.0*PI);
+              x1 = sin(x_grid[2*d+1]*2.0*KPI)*cos(x_grid[2*d]*2.0*KPI);
+              x2 = sin(x_grid[2*d+1]*2.0*KPI)*sin(x_grid[2*d]*2.0*KPI);
+              x3 = cos(x_grid[2*d+1]*2.0*KPI);
               temp = sqrt(x1*x1)+sqrt(x2*x2)+sqrt(x3*x3);
               f_grid[d] = 1.0/(temp);
             }
@@ -838,9 +837,9 @@ int main (int argc, char **argv)
             {
               for (d = 0; d < m_compare; d++)
               {
-                x1 = sin(x_compare[2*d+1]*2.0*PI)*cos(x_compare[2*d]*2.0*PI);
-                x2 = sin(x_compare[2*d+1]*2.0*PI)*sin(x_compare[2*d]*2.0*PI);
-                x3 = cos(x_compare[2*d+1]*2.0*PI);
+                x1 = sin(x_compare[2*d+1]*2.0*KPI)*cos(x_compare[2*d]*2.0*KPI);
+                x2 = sin(x_compare[2*d+1]*2.0*KPI)*sin(x_compare[2*d]*2.0*KPI);
+                x3 = cos(x_compare[2*d+1]*2.0*KPI);
                 temp = sqrt(x1*x1)+sqrt(x2*x2)+sqrt(x3*x3);
                 f_compare[d] = 1.0/(temp);
               }
@@ -853,9 +852,9 @@ int main (int argc, char **argv)
           case FUNCTION_F5:
             for (d = 0; d < m_total; d++)
             {
-              x1 = sin(x_grid[2*d+1]*2.0*PI)*cos(x_grid[2*d]*2.0*PI);
-              x2 = sin(x_grid[2*d+1]*2.0*PI)*sin(x_grid[2*d]*2.0*PI);
-              x3 = cos(x_grid[2*d+1]*2.0*PI);
+              x1 = sin(x_grid[2*d+1]*2.0*KPI)*cos(x_grid[2*d]*2.0*KPI);
+              x2 = sin(x_grid[2*d+1]*2.0*KPI)*sin(x_grid[2*d]*2.0*KPI);
+              x3 = cos(x_grid[2*d+1]*2.0*KPI);
               temp = sqrt(x1*x1)+sqrt(x2*x2)+sqrt(x3*x3);
               f_grid[d] = 0.1*sin(1+temp)*sin(1+temp);
             }
@@ -863,9 +862,9 @@ int main (int argc, char **argv)
             {
               for (d = 0; d < m_compare; d++)
               {
-                x1 = sin(x_compare[2*d+1]*2.0*PI)*cos(x_compare[2*d]*2.0*PI);
-                x2 = sin(x_compare[2*d+1]*2.0*PI)*sin(x_compare[2*d]*2.0*PI);
-                x3 = cos(x_compare[2*d+1]*2.0*PI);
+                x1 = sin(x_compare[2*d+1]*2.0*KPI)*cos(x_compare[2*d]*2.0*KPI);
+                x2 = sin(x_compare[2*d+1]*2.0*KPI)*sin(x_compare[2*d]*2.0*KPI);
+                x3 = cos(x_compare[2*d+1]*2.0*KPI);
                 temp = sqrt(x1*x1)+sqrt(x2*x2)+sqrt(x3*x3);
                 f_compare[d] = 0.1*sin(1+temp)*sin(1+temp);
               }
@@ -884,7 +883,7 @@ int main (int argc, char **argv)
               }
               else
               {
-                f_grid[d] = 1.0/(sqrt(1+3*cos(2.0*PI*x_grid[2*d+1])*cos(2.0*PI*x_grid[2*d+1])));
+                f_grid[d] = 1.0/(sqrt(1+3*cos(2.0*KPI*x_grid[2*d+1])*cos(2.0*KPI*x_grid[2*d+1])));
               }
             }
             if (mode == RANDOM)
@@ -897,7 +896,7 @@ int main (int argc, char **argv)
                 }
                 else
                 {
-                  f_compare[d] = 1.0/(sqrt(1+3*cos(2.0*PI*x_compare[2*d+1])*cos(2.0*PI*x_compare[2*d+1])));
+                  f_compare[d] = 1.0/(sqrt(1+3*cos(2.0*KPI*x_compare[2*d+1])*cos(2.0*KPI*x_compare[2*d+1])));
                 }
               }
             }
