@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2015 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -16,9 +16,11 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: infft.h 3896 2012-10-10 12:19:26Z tovo $ */
+/* $Id$ */
 
-/* NFFT internal header file */
+/*! \file infft3.h
+ *  \brief Internal header file for auxiliary definitions and functions.
+ */
 #ifndef __INFFT_H__
 #define __INFFT_H__
 
@@ -52,26 +54,49 @@
 
 #include "ticks.h"
 
+/**
+ * @defgroup nfftutil Util - Auxilliary functions
+ * @{
+ *
+ * This module implements frequently used utility functions.
+ * In particular, this includes simple measurement of resources, evaluation of
+ * window functions, vector routines for basic linear algebra tasks, and
+ * computation of weights for the inverse transforms.
+ *
+ */
+
 /* Determine precision and name-mangling scheme. */
 #define CONCAT(prefix, name) prefix ## name
 #if defined(NFFT_SINGLE)
 typedef float R;
 typedef float _Complex C;
 #define Y(name) CONCAT(nfftf_,name)
-#define Z(name) CONCAT(fftwf_,name)
+#define FFTW(name) CONCAT(fftwf_,name)
+#define NFFT(name) CONCAT(nfftf_,name)
+#define NFCT(name) CONCAT(nfctf_,name)
+#define NFST(name) CONCAT(nfstf_,name)
 #define NFSFT(name) CONCAT(nfsftf_,name)
+#define SOLVER(name) CONCAT(solverf_,name)
 #elif defined(NFFT_LDOUBLE)
 typedef long double R;
 typedef long double _Complex C;
 #define Y(name) CONCAT(nfftl_,name)
-#define Z(name) CONCAT(fftwl_,name)
+#define FFTW(name) CONCAT(fftwl_,name)
+#define NFFT(name) CONCAT(nfftl_,name)
+#define NFCT(name) CONCAT(nfctl_,name)
+#define NFST(name) CONCAT(nfstl_,name)
 #define NFSFT(name) CONCAT(nfsftl_,name)
+#define SOLVER(name) CONCAT(solverl_,name)
 #else
 typedef double R;
 typedef double _Complex C;
 #define Y(name) CONCAT(nfft_,name)
-#define Z(name) CONCAT(fftw_,name)
+#define FFTW(name) CONCAT(fftw_,name)
+#define NFFT(name) CONCAT(nfft_,name)
+#define NFCT(name) CONCAT(nfct_,name)
+#define NFST(name) CONCAT(nfst_,name)
 #define NFSFT(name) CONCAT(nfsft_,name)
+#define SOLVER(name) CONCAT(solver_,name)
 #endif
 #define X(name) Y(name)
 
@@ -85,12 +110,31 @@ typedef double _Complex C;
 #endif
 #define DK(name, value) const R name = K(value)
 
+#if defined __CYGWIN32__ && !defined __CYGWIN__
+   /* For backwards compatibility with Cygwin b19 and
+      earlier, we define __CYGWIN__ here, so that
+      we can rely on checking just for that macro. */
+#  define __CYGWIN__  __CYGWIN32__
+#endif
+
+#if defined _WIN32 && !defined __CYGWIN__
+   /* Use Windows separators on all _WIN32 defining
+      environments, except Cygwin. */
+#  define SEP "\\"
+#endif
+#ifndef SEP
+   /* Assume that not having this is an indicator that all
+      are missing. */
+#  define SEP "/"
+#endif /* !DIR_SEPARATOR_CHAR */
+
 /* Integral type large enough to contain a stride (what ``int'' should have been
  * in the first place) */
 typedef ptrdiff_t INT;
 
 #define KPI K(3.1415926535897932384626433832795028841971693993751)
 #define K2PI K(6.2831853071795864769252867665590057683943387987502)
+#define K4PI K(12.5663706143591729538505735331180115367886775975004)
 #define KE K(2.7182818284590452353602874713526624977572470937000)
 
 #define IF(x,a,b) ((x)?(a):(b))
@@ -101,17 +145,28 @@ typedef ptrdiff_t INT;
 #define SIGN(a) (((a)>=0)?1:-1)
 #define SIGNF(a) IF((a)<K(0.0),K(-1.0),K(1.0))
 
+/* Size of array. */
+#define SIZE(x) sizeof(x)/sizeof(x[0])
+
+/** Swap two vectors. */
+#define CSWAP(x,y) {C* NFFT_SWAP_temp__; \
+  NFFT_SWAP_temp__=(x); (x)=(y); (y)=NFFT_SWAP_temp__;}
+
+/** Swap two vectors. */
+#define RSWAP(x,y) {R* NFFT_SWAP_temp__; NFFT_SWAP_temp__=(x); \
+  (x)=(y); (y)=NFFT_SWAP_temp__;}
+
 /* macros for window functions */
 
 #if defined(DIRAC_DELTA)
-  #define PHI_HUT(k,d) K(1.0)
-  #define PHI(x,d) IF(FABS((x)) < K(10E-8),K(1.0),K(0.0))
+  #define PHI_HUT(n,k,d) K(1.0)
+  #define PHI(n,x,d) IF(FABS((x)) < K(10E-8),K(1.0),K(0.0))
   #define WINDOW_HELP_INIT(d)
   #define WINDOW_HELP_FINALIZE
   #define WINDOW_HELP_ESTIMATE_m 0
 #elif defined(GAUSSIAN)
-  #define PHI_HUT(k,d) ((R)EXP(-(POW(KPI*(k)/ths->n[d],K(2.0))*ths->b[d])))
-  #define PHI(x,d) ((R)EXP(-POW((x)*((R)ths->n[d]),K(2.0)) / \
+  #define PHI_HUT(n,k,d) ((R)EXP(-(POW(KPI*(k)/n,K(2.0))*ths->b[d])))
+  #define PHI(n,x,d) ((R)EXP(-POW((x)*((R)n),K(2.0)) / \
     ths->b[d])/SQRT(KPI*ths->b[d]))
   #define WINDOW_HELP_INIT \
     { \
@@ -122,56 +177,81 @@ typedef ptrdiff_t INT;
           (K(2.0)*ths->sigma[WINDOW_idx] - K(1.0)) * (((R)ths->m) / KPI); \
     }
   #define WINDOW_HELP_FINALIZE {Y(free)(ths->b);}
-  #define WINDOW_HELP_ESTIMATE_m 12
+#if defined(NFFT_LDOUBLE)
+  #define WINDOW_HELP_ESTIMATE_m 17
+#elif defined(NFFT_SINGLE)
+  #define WINDOW_HELP_ESTIMATE_m 5
+#else
+  #define WINDOW_HELP_ESTIMATE_m 13
+#endif
 #elif defined(B_SPLINE)
-  #define PHI_HUT(k,d) ((R)(((k) == 0) ? K(1.0) / ths->n[(d)] : \
-    POW(SIN((k) * KPI / ths->n[(d)]) / ((k) * KPI / ths->n[(d)]), \
-      K(2.0) * ths->m)/ths->n[(d)]))
-  #define PHI(x,d) (Y(bspline)(2*ths->m,((x)*ths->n[(d)]) + \
-    (R)ths->m,ths->spline_coeffs) / ths->n[(d)])
+  #define PHI_HUT(n,k,d) ((R)(((k) == 0) ? K(1.0) / n : \
+    POW(SIN((k) * KPI / n) / ((k) * KPI / n), \
+      K(2.0) * ths->m)/n))
+  #define PHI(n,x,d) (Y(bspline)(2*ths->m,((x)*n) + \
+    (R)ths->m,ths->spline_coeffs) / n)
   #define WINDOW_HELP_INIT \
     { \
       ths->spline_coeffs= (R*)Y(malloc)(2*ths->m*sizeof(R)); \
     }
   #define WINDOW_HELP_FINALIZE {Y(free)(ths->spline_coeffs);}
+#if defined(NFFT_LDOUBLE)
   #define WINDOW_HELP_ESTIMATE_m 11
+#elif defined(NFFT_SINGLE)
+  #define WINDOW_HELP_ESTIMATE_m 11
+#else
+  #define WINDOW_HELP_ESTIMATE_m 11
+#endif
 #elif defined(SINC_POWER)
-  #define PHI_HUT(k,d) (Y(bspline)(2 * ths->m, (K(2.0) * ths->m*(k)) / \
-    ((K(2.0) * ths->sigma[(d)] - 1) * ths->n[(d)] / \
+  #define PHI_HUT(n,k,d) (Y(bspline)(2 * ths->m, (K(2.0) * ths->m*(k)) / \
+    ((K(2.0) * ths->sigma[(d)] - 1) * n / \
       ths->sigma[(d)]) + (R)ths->m, ths->spline_coeffs))
-  #define PHI(x,d) ((R)(ths->n[(d)] / ths->sigma[(d)] * \
+  #define PHI(n,x,d) ((R)(n / ths->sigma[(d)] * \
     (K(2.0) * ths->sigma[(d)] - K(1.0))/ (K(2.0)*ths->m) * \
-    POW(Y(sinc)(KPI * ths->n[(d)] / ths->sigma[(d)] * (x) * \
+    POW(Y(sinc)(KPI * n / ths->sigma[(d)] * (x) * \
     (K(2.0) * ths->sigma[(d)] - K(1.0)) / (K(2.0)*ths->m)) , 2*ths->m) / \
-    ths->n[(d)]))
+    n))
   #define WINDOW_HELP_INIT \
     { \
       ths->spline_coeffs= (R*)Y(malloc)(2 * ths->m * sizeof(R)); \
     }
   #define WINDOW_HELP_FINALIZE {Y(free)(ths->spline_coeffs);}
-  #define WINDOW_HELP_ESTIMATE_m 9
+#if defined(NFFT_LDOUBLE)
+  #define WINDOW_HELP_ESTIMATE_m 13
+#elif defined(NFFT_SINGLE)
+  #define WINDOW_HELP_ESTIMATE_m 11
+#else
+  #define WINDOW_HELP_ESTIMATE_m 11
+#endif
 #else /* Kaiser-Bessel is the default. */
-  #define PHI_HUT(k,d) ((R)Y(bessel_i0)(ths->m * SQRT(\
-    POW((R)(ths->b[d]), K(2.0)) - POW(K(2.0) * KPI * (k) / ths->n[d], K(2.0)))))
-  #define PHI(x,d) ((R)((POW((R)(ths->m), K(2.0))\
-    -POW((x)*ths->n[d],K(2.0))) > 0)? \
-    SINH(ths->b[d] * SQRT(POW((R)(ths->m),K(2.0)) - \
-    POW((x)*ths->n[d],K(2.0))))/(KPI*SQRT(POW((R)(ths->m),K(2.0)) - \
-    POW((x)*ths->n[d],K(2.0)))) : (((POW((R)(ths->m),K(2.0)) - \
-    POW((x)*ths->n[d],K(2.0))) < 0)? SIN(ths->b[d] * \
-    SQRT(POW(ths->n[d]*(x),K(2.0)) - POW((R)(ths->m), K(2.0)))) / \
-    (KPI*SQRT(POW(ths->n[d]*(x),K(2.0)) - POW((R)(ths->m),K(2.0)))):K(1.0)))
+  #define PHI_HUT(n,k,d) (Y(bessel_i0)((R)(ths->m) * SQRT(ths->b[d] * ths->b[d] - (K(2.0) * KPI * (R)(k) / (R)(n)) * (K(2.0) * KPI * (R)(k) / (R)(n)))))
+  #define PHI(n,x,d) (  (((R)(ths->m) * (R)(ths->m) - (x) * (R)(n) * (x) * (R)(n)) > K(0.0)) \
+                      ?   SINH(ths->b[d] * SQRT((R)(ths->m) * (R)(ths->m) - (x) * (R)(n) * (x) * (R)(n))) \
+                        / (KPI * SQRT((R)(ths->m) * (R)(ths->m) - (x) * (R)(n) * (x) * (R)(n))) \
+                      :   ((((R)(ths->m) * (R)(ths->m) - (x) * (R)(n) * (x) * (R)(n)) < K(0.0)) \
+                        ?   SIN(ths->b[d] * SQRT((x) * (R)(n) * (x) * (R)(n) - (R)(ths->m) * (R)(ths->m))) \
+                          / (KPI * SQRT((x) * (R)(n) * (x) * (R)(n) - (R)(ths->m) * (R)(ths->m))) \
+                        : ths->b[d] / KPI))
   #define WINDOW_HELP_INIT \
     { \
       int WINDOW_idx; \
-      ths->b = (R*) Y(malloc)(ths->d*sizeof(R)); \
+      ths->b = (R*) Y(malloc)((size_t)(ths->d) * sizeof(R)); \
       for (WINDOW_idx = 0; WINDOW_idx < ths->d; WINDOW_idx++) \
-        ths->b[WINDOW_idx] = ((R)KPI*(K(2.0)-K(1.0) / ths->sigma[WINDOW_idx])); \
+        ths->b[WINDOW_idx] = (KPI * (K(2.0) - K(1.0) / ths->sigma[WINDOW_idx])); \
   }
   #define WINDOW_HELP_FINALIZE {Y(free)(ths->b);}
-  #define WINDOW_HELP_ESTIMATE_m 6
+  #if defined(NFFT_LDOUBLE)
+    #define WINDOW_HELP_ESTIMATE_m 9
+  #elif defined(NFFT_SINGLE)
+    #define WINDOW_HELP_ESTIMATE_m 4
+  #else
+    #define WINDOW_HELP_ESTIMATE_m 8
+  #endif
 #endif
 
+/* window.c */
+INT Y(m2K)(const INT m);
+
 #if defined(NFFT_LDOUBLE)
 #if HAVE_DECL_COPYSIGNL == 0
 extern long double copysignl(long double, long double);
@@ -1223,39 +1303,36 @@ extern double _Complex catanh(double _Complex z);
 
 /* format strings */
 #if defined(NFFT_LDOUBLE)
-#  define FE "LE"
-#  define FE_ "% 36.32LE"
-#  define FFI "%Lf"
+#  define __FGS__ "Lg"
+#  define __FES__ "LE"
+#  define __FE__ "% 36.32LE"
+#  define __FI__ "%Lf"
+#  define __FIS__ "Lf"
+#  define __FR__ "%La"
 #elif defined(NFFT_SINGLE)
-#  define FE "E"
-#  define FE_ "% 12.8E"
-#  define FFI "%f"
+#  define __FGS__ "g"
+#  define __FES__ "E"
+#  define __FE__ "% 12.8E"
+#  define __FI__ "%f"
+#  define __FIS__ "f"
+#  define __FR__ "%a"
 #else
-#  define FE "lE"
-#  define FE_ "% 20.16lE"
-#  define FFI "%lf"
+#  define __FGS__ "lg"
+#  define __FES__ "lE"
+#  define __FE__ "% 20.16lE"
+#  define __FI__ "%lf"
+#  define __FIS__ "lf"
+#  define __FR__ "%la"
 #endif
 
 #define TRUE 1
 #define FALSE 0
 
+#define __D__ "%td"
+
 /** Dummy use of unused parameters to silence compiler warnings */
 #define UNUSED(x) (void)x
 
-extern void nfft_assertion_failed(const char *s, int line, const char *file);
-
-/* always check */
-#define CK(ex) \
-  (void)((ex) || (nfft_assertion_failed(#ex, __LINE__, __FILE__), 0))
-
-#ifdef NFFT_DEBUG
-  /* check only if debug enabled */
-  #define A(ex) \
-    (void)((ex) || (nfft_assertion_failed(#ex, __LINE__, __FILE__), 0))
-#else
-  #define A(ex) /* nothing */
-#endif
-
 #ifdef HAVE_ALLOCA
   /* Use alloca if available. */
   #ifndef alloca
@@ -1295,12 +1372,12 @@ extern void nfft_assertion_failed(const char *s, int line, const char *file);
 #else /* ! HAVE_ALLOCA */
   /* Use malloc instead of alloca. So we allocate memory on the heap instead of
    * on the stack which is slower. */
-  #define STACK_MALLOC(T, p, x) p = (T)nfft_malloc(x)
-  #define STACK_FREE(x) nfft_free(x)
+  #define STACK_MALLOC(T, p, x) p = (T)Y(malloc)(x)
+  #define STACK_FREE(x) Y(free)(x)
 #endif /* ! HAVE_ALLOCA */
 
 /** Return number of elapsed seconds between two time points. */
-double nfft_elapsed_seconds(ticks t1, ticks t0);
+R Y(elapsed_seconds)(ticks t1, ticks t0);
 
 /** Dummy use of unused parameters to silence compiler warnings */
 #define UNUSED(x) (void)x
@@ -1329,7 +1406,7 @@ double nfft_elapsed_seconds(ticks t1, ticks t0);
 
 #define TOC(a)                                                                \
       MEASURE_TIME_t1 = getticks();                                           \
-      MEASURE_TIME_tt = nfft_elapsed_seconds(MEASURE_TIME_t1,MEASURE_TIME_t0);\
+      MEASURE_TIME_tt = Y(elapsed_seconds)(MEASURE_TIME_t1,MEASURE_TIME_t0);\
       ths->MEASURE_TIME_t[(a)]+=MEASURE_TIME_tt;                              \
     }                                                                         \
   ths->MEASURE_TIME_t[(a)]/=MEASURE_TIME_r;                                   \
@@ -1350,40 +1427,125 @@ double nfft_elapsed_seconds(ticks t1, ticks t0);
 /* sinc.c: */
 
 /* Sinus cardinalis. */
-R X(sinc)(R x);
+R Y(sinc)(R x);
 
 /* lambda.c: */
 
 /* lambda(z, eps) = gamma(z + eps) / gamma(z + 1) */
-R X(lambda)(R z, R eps);
+R Y(lambda)(R z, R eps);
 
 /* lambda2(mu, nu) = sqrt(gamma(mu + nu + 1) / (gamma(mu + 1) * gamma(nu + 1))) */
-R X(lambda2)(R mu, R nu);
+R Y(lambda2)(R mu, R nu);
 
 /* bessel_i0.c: */
-R X(bessel_i0)(R x);
+R Y(bessel_i0)(R x);
+
+/* bspline.c: */
+R Y(bspline)(const INT, const R x, R*);
 
 /* float.c: */
-typedef enum {NFFT_EPSILON = 0, NFFT_SAFE_MIN = 1, NFFT_BASE = 2,
+typedef enum {NFFT_EPSILON = 0, NFFT_SAFE__MIN = 1, NFFT_BASE = 2,
   NFFT_PRECISION = 3, NFFT_MANT_DIG = 4, NFFT_FLTROUND = 5, NFFT_E_MIN = 6,
   NFFT_R_MIN = 7, NFFT_E_MAX = 8, NFFT_R_MAX = 9} float_property;
 
-R X(float_property)(float_property);
+R Y(float_property)(float_property);
+R Y(prod_real)(R *vec, INT d);
 
 /* int.c: */
-int X(exp2i)(const int a);
-int X(log2i)(const int m);
-int X(next_power_of_2)(const int N);
-void X(next_power_of_2_exp)(const int N, int *N2, int *t);
+INT Y(log2i)(const INT m);
+void Y(next_power_of_2_exp)(const INT N, INT *N2, INT *t);
 
 /* error.c: */
-R X(error_l_infty_complex)(const C *x, const C *y, const INT n);
-/* not used */ R X(error_l_infty_double)(const R *x, const R *y, const INT n);
-R X(error_l_infty_1_complex)(const C *x, const C *y, const INT n,
-  const C *z, const INT m);
-/* not used */ R X(error_l_infty_1_double)(const R *x, const R *y, const INT n, const R *z,
+/* not used */ R Y(error_l_infty_double)(const R *x, const R *y, const INT n);
+/* not used */ R Y(error_l_infty_1_double)(const R *x, const R *y, const INT n, const R *z,
   const INT m);
-R X(error_l_2_complex)(const C *x, const C *y, const INT n);
-/* not used */ R X(error_l_2_double)(const R *x, const R *y, const INT n);
+R Y(error_l_2_complex)(const C *x, const C *y, const INT n);
+/* not used */ R Y(error_l_2_double)(const R *x, const R *y, const INT n);
+
+/* sort.c: */
+void Y(sort_node_indices_radix_msdf)(INT n, INT *keys0, INT *keys1, INT rhigh);
+void Y(sort_node_indices_radix_lsdf)(INT n, INT *keys0, INT *keys1, INT rhigh);
+
+/* assert.c */
+void Y(assertion_failed)(const char *s, int line, const char *file);
+
+/* vector1.c */
+/** Computes the inner/dot product \f$x^H x\f$. */
+R Y(dot_double)(R *x, INT n);
+/** Computes the weighted inner/dot product \f$x^H (w \odot x)\f$. */
+R Y(dot_w_complex)(C *x, R *w, INT n);
+/** Computes the weighted inner/dot product \f$x^H (w \odot x)\f$. */
+R Y(dot_w_double)(R *x, R *w, INT n);
+/** Computes the weighted inner/dot product \f$x^H (w\odot w2\odot w2 \odot x)\f$. */
+R Y(dot_w_w2_complex)(C *x, R *w, R *w2, INT n);
+/** Computes the weighted inner/dot product \f$x^H (w2\odot w2 \odot x)\f$. */
+R Y(dot_w2_complex)(C *x, R *w2, INT n);
+
+/* vector2.c */
+/** Copies \f$x \leftarrow y\f$. */
+void Y(cp_complex)(C *x, C *y, INT n);
+/** Copies \f$x \leftarrow y\f$. */
+void Y(cp_double)(R *x, R *y, INT n);
+/** Copies \f$x \leftarrow a y\f$. */
+void Y(cp_a_complex)(C *x, R a, C *y, INT n);
+/** Copies \f$x \leftarrow a y\f$. */
+void Y(cp_a_double)(R *x, R a, R *y, INT n);
+/** Copies \f$x \leftarrow w\odot y\f$. */
+void Y(cp_w_complex)(C *x, R *w, C *y, INT n);
+/** Copies \f$x \leftarrow w\odot y\f$. */
+void Y(cp_w_double)(R *x, R *w, R *y, INT n);
+
+/* vector3.c */
+/** Updates \f$x \leftarrow a x + y\f$. */
+void Y(upd_axpy_double)(R *x, R a, R *y, INT n);
+/** Updates \f$x \leftarrow x + a y\f$. */
+void Y(upd_xpay_complex)(C *x, R a, C *y, INT n);
+/** Updates \f$x \leftarrow x + a y\f$. */
+void Y(upd_xpay_double)(R *x, R a, R *y, INT n);
+/** Updates \f$x \leftarrow a x + b y\f$. */
+void Y(upd_axpby_complex)(C *x, R a, C *y, R b, INT n);
+/** Updates \f$x \leftarrow a x + b y\f$. */
+void Y(upd_axpby_double)(R *x, R a, R *y, R b, INT n);
+/** Updates \f$x \leftarrow x + a w\odot y\f$. */
+void Y(upd_xpawy_complex)(C *x, R a, R *w, C *y, INT n);
+/** Updates \f$x \leftarrow x + a w\odot y\f$. */
+void Y(upd_xpawy_double)(R *x, R a, R *w, R *y, INT n);
+/** Updates \f$x \leftarrow a x +  w\odot y\f$. */
+void Y(upd_axpwy_complex)(C *x, R a, R *w, C *y, INT n);
+/** Updates \f$x \leftarrow a x +  w\odot y\f$. */
+void Y(upd_axpwy_double)(R *x, R a, R *w, R *y, INT n);
+
+/* voronoi.c */
+void Y(voronoi_weights_1d)(R *w, R *x, const INT M);
+
+/* damp.c */
+/**
+ * Compute damping factor for modified Fejer kernel:
+ * /f$\frac{2}{N}\left(1-\frac{\left|2k+1\right|}{N}\right)/f$
+ */
+R Y(modified_fejer)(const INT N, const INT kk);
+/** Compute damping factor for modified Jackson kernel. */
+R Y(modified_jackson2)(const INT N, const INT kk);
+/** Compute damping factor for modified generalised Jackson kernel. */
+R Y(modified_jackson4)(const INT N, const INT kk);
+/** Compute damping factor for modified Sobolev kernel. */
+R Y(modified_sobolev)(const R mu, const INT kk);
+/** Comput damping factor for modified multiquadric kernel. */
+R Y(modified_multiquadric)(const R mu, const R c, const INT kk);
+
+/* always check */
+#define CK(ex) \
+  (void)((ex) || (Y(assertion_failed)(#ex, __LINE__, __FILE__), 0))
+
+#ifdef NFFT_DEBUG
+  /* check only if debug enabled */
+  #define A(ex) \
+    (void)((ex) || (Y(assertion_failed)(#ex, __LINE__, __FILE__), 0))
+#else
+  #define A(ex) /* nothing */
+#endif
+
+/** @}
+ */
 
 #endif
