@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2015 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -16,14 +16,11 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: nfft3.h 3896 2012-10-10 12:19:26Z tovo $ */
+/* $Id$ */
 
 #ifndef __NFFT3_H__
 #define __NFFT3_H__
 
-/* module configuration */
-#include "nfft3conf.h"
-
 /* fftw_complex */
 #include <fftw3.h>
 
@@ -51,42 +48,52 @@ extern "C"
 #  define NFFT_EXTERN extern
 #endif
 
-/* our own memory allocation and exit functions */
-NFFT_EXTERN void *nfft_malloc(size_t n);
-NFFT_EXTERN void nfft_free(void *p);
-NFFT_EXTERN void nfft_die(char *s);
+/* Integral type large enough to contain a stride (what ``int'' should have been
+ * in the first place) */
+typedef ptrdiff_t NFFT_INT;
 
-/* You can replace the hooks with your own, functions if necessary. We need this
- * for the Matlab interfaces etc. */
-typedef void *(*nfft_malloc_type_function) (size_t n);
-typedef void  (*nfft_free_type_function) (void *p);
-typedef void  (*nfft_die_type_function) (const char *errString);
-NFFT_EXTERN nfft_malloc_type_function nfft_malloc_hook;
-NFFT_EXTERN nfft_free_type_function nfft_free_hook;
-NFFT_EXTERN nfft_die_type_function nfft_die_hook;
-
-/* members inherited by all plans */
+/* Members inherited by all plans. */
 #define MACRO_MV_PLAN(RC) \
-  int N_total; /**< Total number of Fourier coefficients */\
-  int M_total; /**< Total number of samples */\
-  RC *f_hat; /**< Vector of Fourier coefficients, size is N_total * sizeof(RC) */\
-  RC *f; /**< Vector of samples, size is M_total * sizeof(RC) */\
-  void (*mv_trafo)(void*); /**< Pointer to the own transform */\
-  void (*mv_adjoint)(void*); /**< Pointer to the own adjoint */
+  NFFT_INT N_total; /**< Total number of Fourier coefficients. */\
+  NFFT_INT M_total; /**< Total number of samples. */\
+  RC *f_hat; /**< Fourier coefficients. */\
+  RC *f; /**< Samples. */\
+  void (*mv_trafo)(void*); /**< Transform. */\
+  void (*mv_adjoint)(void*); /**< Adjoint transform. */
 
 /* nfft */
 
-/* name mangling macros */
+/* Name mangling macros. */
 #define NFFT_MANGLE_DOUBLE(name) NFFT_CONCAT(nfft_, name)
 #define NFFT_MANGLE_FLOAT(name) NFFT_CONCAT(nfftf_, name)
 #define NFFT_MANGLE_LONG_DOUBLE(name) NFFT_CONCAT(nfftl_, name)
 
-/* huge second-order macro that defines prototypes for all nfft API functions.
+#define NFFT_DEFINE_MALLOC_API(X) \
+/* our own memory allocation and exit functions */ \
+NFFT_EXTERN void *X(malloc)(size_t n); \
+NFFT_EXTERN void X(free)(void *p); \
+NFFT_EXTERN void X(die)(const char *s); \
+\
+/* You can replace the hooks with your own functions, if necessary. We */ \
+/* need this for the Matlab interface. */ \
+typedef void *(*X(malloc_type_function)) (size_t n); \
+typedef void  (*X(free_type_function)) (void *p); \
+typedef void  (*X(die_type_function)) (const char *errString); \
+NFFT_EXTERN X(malloc_type_function) X(malloc_hook); \
+NFFT_EXTERN X(free_type_function) X(free_hook); \
+NFFT_EXTERN X(die_type_function) X(die_hook);
+
+/* Nfft module API. */
+NFFT_DEFINE_MALLOC_API(NFFT_MANGLE_FLOAT)
+NFFT_DEFINE_MALLOC_API(NFFT_MANGLE_DOUBLE)
+NFFT_DEFINE_MALLOC_API(NFFT_MANGLE_LONG_DOUBLE)
+
+/* Macro to define prototypes for all NFFT API functions.
  * We expand this macro for each supported precision.
- *   X: nfft name-mangling macro
- *   Y: fftw name-mangling macro
- *   R: real data type
- *   C: complex data type
+ *   X: NFFT name-mangling macro
+ *   Y: FFTW name-mangling macro
+ *   R: Real float type
+ *   C: Complex float type
  */
 #define NFFT_DEFINE_API(X,Y,R,C) \
 \
@@ -100,48 +107,50 @@ typedef struct \
   MACRO_MV_PLAN(R) \
 } X(mv_plan_double); \
 \
+/** data structure for an NFFT (nonequispaced fast Fourier transform) plan with R precision */ \
 typedef struct\
 {\
   MACRO_MV_PLAN(C)\
 \
-  int d; /**< dimension aka rank */\
-  int *N; /**< multi-bandwidth */\
-  R *sigma; /**< oversampling-factor */\
-  int *n; /**< FFTW length, equal to sigma*N, default is the power of 2 such
-               that \f$2\le\sigma<4\f$ */\
-  int n_total; /**< Total size of FFTW */\
-  int m; /**< Cut-off parameter of the window function, default value is
-               6 (KAISER_BESSEL),
-               9 (SINC_POWER),
-               11 (B_SPLINE),
-               12 (GAUSSIAN) */\
-  R *b; /**< Shape parameter of the window function */\
-  int K; /**< Number of equispaced samples of the window function for \ref
-              PRE_LIN_PSI */\
+  NFFT_INT d; /**< Dimension (rank). */\
+  NFFT_INT *N; /**< Multi-bandwidth. */\
+  R *sigma; /**< Oversampling factor. */\
+  NFFT_INT *n; /**< Length of FFTW transforms. This is equal to sigma*N. The default
+               is to use a power of two that satifies  \f$2\le\sigma<4\f$. */\
+  NFFT_INT n_total; /**< Combined total length of FFTW transforms. */\
+  NFFT_INT m; /**< Cut-off parameter for window function. Default values for the
+              different window functions are
+              -  6 (KAISER_BESSEL),
+              -  9 (SINC_POWER),
+              - 11 (B_SPLINE),
+              - 12 (GAUSSIAN) */\
+  R *b; /**< Shape parameter for window function */\
+  NFFT_INT K; /**< Number of equispaced samples of window function. Used for flag
+             PRE_LIN_PSI. */\
 \
-  unsigned nfft_flags; /**< Flags for precomputation, (de)allocation, and FFTW
-                            usage, default setting is
-                            PRE_PHI_HUT | PRE_PSI | MALLOC_X | MALLOC_F_HAT |
-                            MALLOC_F | FFTW_INIT | FFT_OUT_OF_PLACE */\
+  unsigned flags; /**< Flags for precomputation, (de)allocation, and FFTW
+                       usage, default setting is PRE_PHI_HUT | PRE_PSI
+                       | MALLOC_X | MALLOC_F_HAT | MALLOC_F | FFTW_INIT
+                       | FFT_OUT_OF_PLACE */\
 \
   unsigned fftw_flags; /**< Flags for the FFTW, default is
                             FFTW_ESTIMATE | FFTW_DESTROY_INPUT */\
 \
-  R *x; /**< Nodes in time/spatial domain, size is \f$dM\f$ doubles */\
+  R *x; /**< Nodes in time/spatial domain, size is \f$dM\f$ R ## s */\
 \
-  double MEASURE_TIME_t[3]; /**< Measured time for each step if MEASURE_TIME is
+  R MEASURE_TIME_t[3]; /**< Measured time for each step if MEASURE_TIME is
     set */\
 \
   /* internal use only */\
-  Y(plan)  my_fftw_plan1; /**< Forward FFTW plan */\
-  Y(plan)  my_fftw_plan2; /**< Backward FFTW plan */\
+  Y(plan) my_fftw_plan1; /**< Forward FFTW plan */\
+  Y(plan) my_fftw_plan2; /**< Backward FFTW plan */\
 \
   R **c_phi_inv; /**< Precomputed data for the diagonal matrix \f$D\f$, size \
     is \f$N_0+\hdots+N_{d-1}\f$ doubles*/\
   R *psi; /**< Precomputed data for the sparse matrix \f$B\f$, size depends
                     on precomputation scheme */\
-  int *psi_index_g; /**< Indices in source/target vector for \ref PRE_FULL_PSI */\
-  int *psi_index_f; /**< Indices in source/target vector for \ref PRE_FULL_PSI */\
+  NFFT_INT *psi_index_g; /**< Indices in source/target vector for \ref PRE_FULL_PSI */\
+  NFFT_INT *psi_index_f; /**< Indices in source/target vector for \ref PRE_FULL_PSI */\
 \
   C *g; /**< Oversampled vector of samples, size is \ref n_total double complex */\
   C *g_hat; /**< Zero-padded vector of Fourier coefficients, size is \ref n_total fftw_complex */\
@@ -150,11 +159,11 @@ typedef struct\
 \
   R *spline_coeffs; /**< Input for de Boor algorithm if B_SPLINE or SINC_POWER is defined */\
 \
-  int *index_x; /**< Index array for nodes x used when flag \ref NFFT_SORT_NODES is set */\
+  NFFT_INT *index_x; /**< Index array for nodes x used when flag \ref NFFT_SORT_NODES is set. */\
 } X(plan); \
 \
-NFFT_EXTERN void X(trafo_direct)(X(plan) *ths);\
-NFFT_EXTERN void X(adjoint_direct)(X(plan) *ths);\
+NFFT_EXTERN void X(trafo_direct)(const X(plan) *ths);\
+NFFT_EXTERN void X(adjoint_direct)(const X(plan) *ths);\
 NFFT_EXTERN void X(trafo)(X(plan) *ths);\
 NFFT_EXTERN void X(trafo_1d)(X(plan) *ths);\
 NFFT_EXTERN void X(trafo_2d)(X(plan) *ths);\
@@ -168,36 +177,38 @@ NFFT_EXTERN void X(init_2d)(X(plan) *ths, int N1, int N2, int M);\
 NFFT_EXTERN void X(init_3d)(X(plan) *ths, int N1, int N2, int N3, int M);\
 NFFT_EXTERN void X(init)(X(plan) *ths, int d, int *N, int M);\
 NFFT_EXTERN void X(init_guru)(X(plan) *ths, int d, int *N, int M, int *n, \
-  int m, unsigned nfft_flags, unsigned fftw_flags);\
+  int m, unsigned flags, unsigned fftw_flags);\
+NFFT_EXTERN void X(init_lin)(X(plan) *ths, int d, int *N, int M, int *n, \
+  int m, int K, unsigned flags, unsigned fftw_flags); \
 NFFT_EXTERN void X(precompute_one_psi)(X(plan) *ths);\
-NFFT_EXTERN void X(precompute_full_psi)(X(plan) *ths);\
 NFFT_EXTERN void X(precompute_psi)(X(plan) *ths);\
+NFFT_EXTERN void X(precompute_full_psi)(X(plan) *ths);\
+NFFT_EXTERN void X(precompute_fg_psi)(X(plan) *ths); \
 NFFT_EXTERN void X(precompute_lin_psi)(X(plan) *ths);\
 NFFT_EXTERN const char* X(check)(X(plan) *ths);\
 NFFT_EXTERN void X(finalize)(X(plan) *ths);
 
-/* nfft api */
+/* Nfft module API. */
 NFFT_DEFINE_API(NFFT_MANGLE_FLOAT,FFTW_MANGLE_FLOAT,float,fftwf_complex)
 NFFT_DEFINE_API(NFFT_MANGLE_DOUBLE,FFTW_MANGLE_DOUBLE,double,fftw_complex)
 NFFT_DEFINE_API(NFFT_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,long double,fftwl_complex)
 
-/* flags for init */
-#define PRE_PHI_HUT      (1U<< 0)
-#define FG_PSI           (1U<< 1)
-#define PRE_LIN_PSI      (1U<< 2)
-#define PRE_FG_PSI       (1U<< 3)
-#define PRE_PSI          (1U<< 4)
-#define PRE_FULL_PSI     (1U<< 5)
-#define MALLOC_X         (1U<< 6)
-#define MALLOC_F_HAT     (1U<< 7)
-#define MALLOC_F         (1U<< 8)
-#define FFT_OUT_OF_PLACE (1U<< 9)
-#define FFTW_INIT        (1U<< 10)
-#define NFFT_SORT_NODES  (1U<< 11)
+/* Flags for init routines. */
+#define PRE_PHI_HUT                (1U<<0)
+#define FG_PSI                     (1U<<1)
+#define PRE_LIN_PSI                (1U<<2)
+#define PRE_FG_PSI                 (1U<<3)
+#define PRE_PSI                    (1U<<4)
+#define PRE_FULL_PSI               (1U<<5)
+#define MALLOC_X                   (1U<<6)
+#define MALLOC_F_HAT               (1U<<7)
+#define MALLOC_F                   (1U<<8)
+#define FFT_OUT_OF_PLACE           (1U<<9)
+#define FFTW_INIT                  (1U<<10)
+#define NFFT_SORT_NODES            (1U<<11)
 #define NFFT_OMP_BLOCKWISE_ADJOINT (1U<<12)
 #define PRE_ONE_PSI (PRE_LIN_PSI| PRE_FG_PSI| PRE_PSI| PRE_FULL_PSI)
 
-
 /* nfct */
 
 /* name mangling macros */
@@ -213,21 +224,24 @@ NFFT_DEFINE_API(NFFT_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,long double,fftw
  *   C: complex data type
  */
 #define NFCT_DEFINE_API(X,Y,R,C) \
+/** data structure for an NFCT (nonequispaced fast cosine transform) plan with R precision */ \
 typedef struct\
 {\
   /* api */\
   MACRO_MV_PLAN(R)\
 \
-  int d; /**< dimension, rank */\
-  int *N; /**< cut-off-frequencies (kernel) */\
-  int *n; /**< length of DCT-I */\
+  NFFT_INT d; /**< dimension, rank */\
+  NFFT_INT *N; /**< cut-off-frequencies (kernel) */\
+  NFFT_INT *n; /**< length of DCT-I */\
+  NFFT_INT n_total; /**< Combined total length of FFTW transforms. */\
   R *sigma; /**< oversampling-factor */\
-  int m; /**< cut-off parameter in time-domain */\
+  NFFT_INT m; /**< cut-off parameter in time-domain */\
 \
-  R nfct_full_psi_eps;\
   R *b; /**< shape parameters */\
+  NFFT_INT K; /**< Number of equispaced samples of window function. Used for flag
+               PRE_LIN_PSI. */\
 \
-  unsigned nfct_flags; /**< flags for precomputation, malloc */\
+  unsigned flags; /**< flags for precomputation, malloc */\
   unsigned fftw_flags; /**< flags for the fftw */\
 \
   R *x; /**< nodes (in time/spatial domain)   */\
@@ -240,9 +254,9 @@ typedef struct\
 \
   R **c_phi_inv; /**< precomputed data, matrix D */\
   R *psi; /**< precomputed data, matrix B */\
-  int size_psi; /**< only for thin B */\
-  int *psi_index_g; /**< only for thin B */\
-  int *psi_index_f; /**< only for thin B */\
+  NFFT_INT size_psi; /**< only for thin B */\
+  NFFT_INT *psi_index_g; /**< only for thin B */\
+  NFFT_INT *psi_index_f; /**< only for thin B */\
 \
   R *g;\
   R *g_hat;\
@@ -257,22 +271,23 @@ NFFT_EXTERN void X(init_2d)(X(plan) *ths_plan, int N0, int N1, int M_total); \
 NFFT_EXTERN void X(init_3d)(X(plan) *ths_plan, int N0, int N1, int N2, int M_total); \
 NFFT_EXTERN void X(init)(X(plan) *ths_plan, int d, int *N, int M_total); \
 NFFT_EXTERN void X(init_guru)(X(plan) *ths_plan, int d, int *N, int M_total, int *n, \
-  int m, unsigned nfct_flags, unsigned fftw_flags); \
-NFFT_EXTERN void X(precompute_psi)(X(plan) *ths_plan); \
+  int m, unsigned flags, unsigned fftw_flags); \
+NFFT_EXTERN void X(precompute_one_psi)(X(plan) *ths);\
+NFFT_EXTERN void X(precompute_psi)(X(plan) *ths);\
+NFFT_EXTERN void X(precompute_full_psi)(X(plan) *ths);\
+NFFT_EXTERN void X(precompute_fg_psi)(X(plan) *ths); \
+NFFT_EXTERN void X(precompute_lin_psi)(X(plan) *ths);\
 NFFT_EXTERN void X(trafo)(X(plan) *ths_plan); \
-NFFT_EXTERN void X(trafo_direct)(X(plan) *ths_plan); \
+NFFT_EXTERN void X(trafo_direct)(const X(plan) *ths_plan); \
 NFFT_EXTERN void X(adjoint)(X(plan) *ths_plan); \
-NFFT_EXTERN void X(adjoint_direct)(X(plan) *ths_plan); \
+NFFT_EXTERN void X(adjoint_direct)(const X(plan) *ths_plan); \
+NFFT_EXTERN const char* X(check)(X(plan) *ths);\
 NFFT_EXTERN void X(finalize)(X(plan) *ths_plan); \
-NFFT_EXTERN R X(phi_hut)(X(plan) *ths_plan, int k, int d); \
-NFFT_EXTERN R X(phi)(X(plan) *ths_plan, R x, int d);
 
-#if defined(HAVE_NFCT)
 /* nfct api */
 NFCT_DEFINE_API(NFCT_MANGLE_FLOAT,FFTW_MANGLE_FLOAT,float,fftwf_complex)
 NFCT_DEFINE_API(NFCT_MANGLE_DOUBLE,FFTW_MANGLE_DOUBLE,double,fftw_complex)
 NFCT_DEFINE_API(NFCT_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,long double,fftwl_complex)
-#endif
 
 /* nfst */
 
@@ -289,21 +304,24 @@ NFCT_DEFINE_API(NFCT_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,long double,fftw
  *   C: complex data type
  */
 #define NFST_DEFINE_API(X,Y,R,C) \
+/** data structure for an NFST (nonequispaced fast sine transform) plan with R precision */ \
 typedef struct\
 {\
   /* api */\
   MACRO_MV_PLAN(R)\
 \
-  int d; /**< dimension, rank */\
-  int *N; /**< bandwidth */\
-  int *n; /**< length of DST-1 */\
+  NFFT_INT d; /**< dimension, rank */\
+  NFFT_INT *N; /**< bandwidth */\
+  NFFT_INT *n; /**< length of DST-I */\
+  NFFT_INT n_total; /**< Combined total length of FFTW transforms. */\
   R *sigma; /**< oversampling-factor */\
-  int m; /**< cut-off parameter in time-domain */\
+  NFFT_INT m; /**< cut-off parameter in time-domain */\
 \
-  R nfst_full_psi_eps;\
   R *b; /**< shape parameters */\
+  NFFT_INT K; /**< Number of equispaced samples of window function. Used for flag
+               PRE_LIN_PSI. */\
 \
-  unsigned nfst_flags; /**< flags for precomputation, malloc */\
+  unsigned flags; /**< flags for precomputation, malloc */\
   unsigned fftw_flags; /**< flags for the fftw */\
 \
   R *x; /**< nodes (in time/spatial domain) */\
@@ -316,9 +334,9 @@ typedef struct\
 \
   R **c_phi_inv; /**< precomputed data, matrix D */\
   R *psi; /**< precomputed data, matrix B */\
-  int size_psi; /**< only for thin B */\
-  int *psi_index_g; /**< only for thin B */\
-  int *psi_index_f; /**< only for thin B */\
+  NFFT_INT size_psi; /**< only for thin B */\
+  NFFT_INT *psi_index_g; /**< only for thin B */\
+  NFFT_INT *psi_index_f; /**< only for thin B */\
 \
   R *g;\
   R *g_hat;\
@@ -326,33 +344,32 @@ typedef struct\
   R *g2; /**< output of fftw */\
 \
   R *spline_coeffs; /**< input for de Boor algorithm, if B_SPLINE or SINC_2m is defined */\
+\
+  R X(full_psi_eps);\
 } X(plan);\
 \
 NFFT_EXTERN void X(init_1d)(X(plan) *ths_plan, int N0, int M_total); \
 NFFT_EXTERN void X(init_2d)(X(plan) *ths_plan, int N0, int N1, int M_total); \
 NFFT_EXTERN void X(init_3d)(X(plan) *ths_plan, int N0, int N1, int N2, int M_total); \
 NFFT_EXTERN void X(init)(X(plan) *ths_plan, int d, int *N, int M_total); \
-NFFT_EXTERN void X(init_m)(X(plan) *ths_plan, int d, int *N, int M_total, int m);\
 NFFT_EXTERN void X(init_guru)(X(plan) *ths_plan, int d, int *N, int M_total, int *n, \
-  int m, unsigned nfst_flags, unsigned fftw_flags); \
-NFFT_EXTERN void X(precompute_psi)(X(plan) *ths_plan); \
+  int m, unsigned flags, unsigned fftw_flags); \
+NFFT_EXTERN void X(precompute_one_psi)(X(plan) *ths);\
+NFFT_EXTERN void X(precompute_psi)(X(plan) *ths);\
+NFFT_EXTERN void X(precompute_full_psi)(X(plan) *ths);\
+NFFT_EXTERN void X(precompute_fg_psi)(X(plan) *ths); \
+NFFT_EXTERN void X(precompute_lin_psi)(X(plan) *ths);\
 NFFT_EXTERN void X(trafo)(X(plan) *ths_plan); \
-NFFT_EXTERN void X(trafo_direct)(X(plan) *ths_plan); \
+NFFT_EXTERN void X(trafo_direct)(const X(plan) *ths_plan); \
 NFFT_EXTERN void X(adjoint)(X(plan) *ths_plan); \
-NFFT_EXTERN void X(adjoint_direct)(X(plan) *ths_plan); \
+NFFT_EXTERN void X(adjoint_direct)(const X(plan) *ths_plan); \
+NFFT_EXTERN const char* X(check)(X(plan) *ths);\
 NFFT_EXTERN void X(finalize)(X(plan) *ths_plan); \
-NFFT_EXTERN void X(full_psi)(X(plan) *ths_plan, R eps); \
-NFFT_EXTERN R X(phi_hut)(X(plan) *ths_plan, int k, int d); \
-NFFT_EXTERN R X(phi)(X(plan) *ths_plan, R x, int d); \
-NFFT_EXTERN int X(fftw_2N)(int n); \
-NFFT_EXTERN int X(fftw_2N_rev)(int n);
 
-#ifdef HAVE_NFST
 /* nfst api */
 NFST_DEFINE_API(NFST_MANGLE_FLOAT,FFTW_MANGLE_FLOAT,float,fftwf_complex)
 NFST_DEFINE_API(NFST_MANGLE_DOUBLE,FFTW_MANGLE_DOUBLE,double,fftw_complex)
 NFST_DEFINE_API(NFST_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,long double,fftwl_complex)
-#endif
 
 /* nnfft */
 
@@ -370,6 +387,7 @@ NFST_DEFINE_API(NFST_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,long double,fftw
  *   C: complex data type
  */
 #define NNFFT_DEFINE_API(X,Y,Z,R,C) \
+/** data structure for an NNFFT (nonequispaced in time and frequency fast Fourier transform) plan with R precision */ \
 typedef struct\
 {\
   /* api */\
@@ -400,6 +418,7 @@ typedef struct\
 } X(plan);\
 \
 NFFT_EXTERN void X(init)(X(plan) *ths_plan, int d, int N_total, int M_total, int *N); \
+NFFT_EXTERN void X(init_1d)(X(plan) *ths_plan, int N, int M_total); \
 NFFT_EXTERN void X(init_guru)(X(plan) *ths_plan, int d, int N_total, int M_total, \
   int *N, int *N1, int m, unsigned nnfft_flags); \
 NFFT_EXTERN void X(trafo_direct)(X(plan) *ths_plan); \
@@ -410,14 +429,13 @@ NFFT_EXTERN void X(precompute_lin_psi)(X(plan) *ths_plan); \
 NFFT_EXTERN void X(precompute_psi)(X(plan) *ths_plan); \
 NFFT_EXTERN void X(precompute_full_psi)(X(plan) *ths_plan); \
 NFFT_EXTERN void X(precompute_phi_hut)(X(plan) *ths_plan); \
+NFFT_EXTERN void X(precompute_one_psi)(X(plan) *ths);\
 NFFT_EXTERN void X(finalize)(X(plan) *ths_plan);
 
-#ifdef HAVE_NNFFT
 /* nnfft api */
 NNFFT_DEFINE_API(NNFFT_MANGLE_FLOAT,FFTW_MANGLE_FLOAT,NFFT_MANGLE_FLOAT,float,fftwf_complex)
 NNFFT_DEFINE_API(NNFFT_MANGLE_DOUBLE,FFTW_MANGLE_DOUBLE,NFFT_MANGLE_DOUBLE,double,fftw_complex)
 NNFFT_DEFINE_API(NNFFT_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LONG_DOUBLE,long double,fftwl_complex)
-#endif
 
 /* additional init flags */
 #define MALLOC_V         (1U<< 11)
@@ -437,6 +455,7 @@ NNFFT_DEFINE_API(NNFFT_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LO
  *   C: complex data type
  */
 #define NSFFT_DEFINE_API(X,Y,Z,R,C) \
+/** data structure for an NSFFT (nonequispaced sparse fast Fourier transform) plan with R precision */ \
 typedef struct\
 {\
   MACRO_MV_PLAN(C)\
@@ -468,12 +487,10 @@ NFFT_EXTERN void X(init_random_nodes_coeffs)(X(plan) *ths); \
 NFFT_EXTERN void X(init)(X(plan) *ths, int d, int J, int M, int m, unsigned flags); \
 NFFT_EXTERN void X(finalize)(X(plan) *ths);
 
-#ifdef HAVE_NSFFT
 /* nsfft api */
 NSFFT_DEFINE_API(NSFFT_MANGLE_FLOAT,FFTW_MANGLE_FLOAT,NFFT_MANGLE_FLOAT,float,fftwf_complex)
 NSFFT_DEFINE_API(NSFFT_MANGLE_DOUBLE,FFTW_MANGLE_DOUBLE,NFFT_MANGLE_DOUBLE,double,fftw_complex)
 NSFFT_DEFINE_API(NSFFT_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LONG_DOUBLE,long double,fftwl_complex)
-#endif
 
 /* additional init flags */
 #define NSDFT            (1U<< 12)
@@ -524,12 +541,10 @@ void X(inh_3d_init_guru)(X(inh_3d_plan) *ths, int *N, int M, int *n, \
   int m, R sigma, unsigned nfft_flags, unsigned fftw_flags); \
 void X(inh_3d_finalize)(X(inh_3d_plan) *ths);
 
-#ifdef HAVE_MRI
   /* mri api */
 MRI_DEFINE_API(MRI_MANGLE_FLOAT,NFFT_MANGLE_FLOAT,float,fftwf_complex)
 MRI_DEFINE_API(MRI_MANGLE_DOUBLE,NFFT_MANGLE_DOUBLE,double,fftw_complex)
 MRI_DEFINE_API(MRI_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LONG_DOUBLE,long double,fftwl_complex)
-#endif
 
 /* nfsft */
 
@@ -546,6 +561,7 @@ MRI_DEFINE_API(MRI_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LONG_DOUBLE,long double,fftwl_
  *   C: complex data type
  */
 #define NFSFT_DEFINE_API(X,Z,R,C) \
+/** data structure for an NFSFT (nonequispaced fast spherical Fourier transform) plan with R precision */ \
 typedef struct\
 {\
   MACRO_MV_PLAN(C)\
@@ -578,12 +594,10 @@ NFFT_EXTERN void X(adjoint)(X(plan)* plan); \
 NFFT_EXTERN void X(finalize)(X(plan) *plan); \
 NFFT_EXTERN void X(precompute_x)(X(plan) *plan);
 
-#ifdef HAVE_NFSFT
 /* nfsft api */
 NFSFT_DEFINE_API(NFSFT_MANGLE_FLOAT,NFFT_MANGLE_FLOAT,float,fftwf_complex)
 NFSFT_DEFINE_API(NFSFT_MANGLE_DOUBLE,NFFT_MANGLE_DOUBLE,double,fftw_complex)
 NFSFT_DEFINE_API(NFSFT_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LONG_DOUBLE,long double,fftwl_complex)
-#endif
 
 /* init flags */
 #define NFSFT_NORMALIZED     (1U << 0)
@@ -638,7 +652,6 @@ NFFT_EXTERN void X(transposed)(X(set) set, const int m, C *x, \
   C *y, const int k_end, const unsigned int flags); \
 NFFT_EXTERN void X(finalize)(X(set) set);
 
-#ifdef HAVE_FPT
 /* fpt api */
 FPT_DEFINE_API(FPT_MANGLE_FLOAT,FFTW_MANGLE_FLOAT,float,fftwf_complex)
 FPT_DEFINE_API(FPT_MANGLE_DOUBLE,FFTW_MANGLE_DOUBLE,double,fftw_complex)
@@ -653,7 +666,6 @@ FPT_DEFINE_API(FPT_MANGLE_LONG_DOUBLE,FFTW_MANGLE_LONG_DOUBLE,long double,fftwl_
 /* transform flags */
 #define FPT_FUNCTION_VALUES     (1U << 5)
 #define FPT_AL_SYMMETRY         (1U << 6)
-#endif
 
 /* nfsoft*/
 
@@ -700,7 +712,6 @@ NFFT_EXTERN void X(adjoint)(X(plan) *plan_nfsoft); \
 NFFT_EXTERN void X(finalize)(X(plan) *plan); \
 NFFT_EXTERN int X(posN)(int n,int m, int B);
 
-#ifdef HAVE_NFSOFT
 /* nfsoft api */
 NFSOFT_DEFINE_API(NFSOFT_MANGLE_FLOAT,NFFT_MANGLE_FLOAT,FPT_MANGLE_FLOAT,float,fftwf_complex)
 NFSOFT_DEFINE_API(NFSOFT_MANGLE_DOUBLE,NFFT_MANGLE_DOUBLE,FPT_MANGLE_DOUBLE,double,fftw_complex)
@@ -732,9 +743,7 @@ NFSOFT_DEFINE_API(NFSOFT_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LONG_DOUBLE,FPT_MANGLE_L
 #define NFSOFT_INDEX_TWO(m,n,l,B) ((B+1)*(B+1)+(B+1)*(B+1)*(m+B)-((m-1)*m*(2*m-1)+(B+1)*(B+2)*(2*B+3))/6)+(posN(n,m,B))+(l-MAX(ABS(m),ABS(n)))
 #define NFSOFT_F_HAT_SIZE(B) (((B)+1)*(4*((B)+1)*((B)+1)-1)/3)
 
-#endif
-
-/*solver */
+/* solver */
 
 /* name mangling macros */
 #define SOLVER_MANGLE_DOUBLE(name) NFFT_CONCAT(solver_, name)
@@ -749,6 +758,7 @@ NFSOFT_DEFINE_API(NFSOFT_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LONG_DOUBLE,FPT_MANGLE_L
  *   C: complex data type
  */
 #define SOLVER_DEFINE_API(X,Y,R,C)\
+/** data structure for an inverse NFFT plan with R precision */ \
 typedef struct\
 {\
   Y(mv_plan_complex) *mv; /**< matrix vector multiplication   */\
@@ -777,6 +787,7 @@ NFFT_EXTERN void X(before_loop_complex)(X(plan_complex)* ths);\
 NFFT_EXTERN void X(loop_one_step_complex)(X(plan_complex) *ths);\
 NFFT_EXTERN void X(finalize_complex)(X(plan_complex) *ths);\
 \
+/** data structure for an inverse NFFT plan with R precision */ \
 typedef struct\
 {\
   Y(mv_plan_double) *mv; /**< matrix vector multiplication   */\
@@ -800,10 +811,10 @@ typedef struct\
 } X(plan_double);\
 \
 NFFT_EXTERN void X(init_advanced_double)(X(plan_double)* ths, Y(mv_plan_double) *mv, unsigned flags);\
-NFFT_EXTERN void X(solver_init_double)(X(plan_double)* ths, Y(mv_plan_double) *mv);\
-NFFT_EXTERN void X(solver_before_loop_double)(X(plan_double)* ths);\
-NFFT_EXTERN void X(solver_loop_one_step_double)(X(plan_double) *ths);\
-NFFT_EXTERN void X(solver_finalize_double)(X(plan_double) *ths);
+NFFT_EXTERN void X(init_double)(X(plan_double)* ths, Y(mv_plan_double) *mv);\
+NFFT_EXTERN void X(before_loop_double)(X(plan_double)* ths);\
+NFFT_EXTERN void X(loop_one_step_double)(X(plan_double) *ths);\
+NFFT_EXTERN void X(finalize_double)(X(plan_double) *ths);
 
 /* solver api */
 SOLVER_DEFINE_API(SOLVER_MANGLE_FLOAT,NFFT_MANGLE_FLOAT,float,fftwf_complex)
@@ -819,6 +830,61 @@ SOLVER_DEFINE_API(SOLVER_MANGLE_LONG_DOUBLE,NFFT_MANGLE_LONG_DOUBLE,long double,
 #define PRECOMPUTE_WEIGHT     (1U<< 5)
 #define PRECOMPUTE_DAMP       (1U<< 6)
 
+/* util */
+
+/* huge second-order macro that defines prototypes for all utility API functions.
+ * We expand this macro for each supported precision.
+ *   Y: nfft name-mangling macro
+ *   R: real data type
+ *   C: complex data type
+ */
+#define NFFT_DEFINE_UTIL_API(Y,R,C) \
+/* rand.c */ \
+R Y(drand48)(void); \
+void Y(srand48)(long int seed); \
+\
+/** Inits a vector of random complex numbers in \f$[0,1]\times[0,1]{\rm i}\f$. \
+ */ \
+void Y(vrand_unit_complex)(C *x, const NFFT_INT n); \
+\
+/** Inits a vector of random double numbers in \f$[-1/2,1/2]\f$. \
+ */ \
+void Y(vrand_shifted_unit_double)(R *x, const NFFT_INT n); \
+\
+void Y(vrand_real)(R *x, const NFFT_INT n, const R a, const R b); \
+\
+/* print.c */ \
+/** Print real vector to standard output. */ \
+void Y(vpr_double)(R *x, const NFFT_INT n, const char *text); \
+\
+/** Print complex vector to standard output. */ \
+void Y(vpr_complex)(C *x, const NFFT_INT n, const char *text); \
+/* thread.c */ \
+NFFT_INT Y(get_num_threads)(void); \
+/* time.c */ \
+R Y(clock_gettime_seconds)(void); \
+/* error.c: */ \
+R Y(error_l_infty_complex)(const C *x, const C *y, const NFFT_INT n); \
+R Y(error_l_infty_1_complex)(const C *x, const C *y, const NFFT_INT n, \
+  const C *z, const NFFT_INT m); \
+/* int.c: */ \
+NFFT_INT Y(exp2i)(const NFFT_INT a); \
+NFFT_INT Y(next_power_of_2)(const NFFT_INT N); \
+/* vector1.c */ \
+/** Computes the inner/dot product \f$x^H x\f$. */ \
+R Y(dot_complex)(C *x, NFFT_INT n); \
+/* vector3.c */ \
+/** Updates \f$x \leftarrow a x + y\f$. */ \
+void Y(upd_axpy_complex)(C *x, R a, C *y, NFFT_INT n); \
+/** Swaps each half over N[d]/2. */ \
+void Y(fftshift_complex)(C *x, NFFT_INT d, NFFT_INT* N); \
+void Y(fftshift_complex_int)(C *x, int d, int* N);
+
+
+NFFT_DEFINE_UTIL_API(NFFT_MANGLE_FLOAT,float,fftwf_complex)
+NFFT_DEFINE_UTIL_API(NFFT_MANGLE_DOUBLE,double,fftw_complex)
+NFFT_DEFINE_UTIL_API(NFFT_MANGLE_LONG_DOUBLE,long double,fftwl_complex)
+
 #ifdef __cplusplus
 }  /* extern "C" */
 #endif /* __cplusplus */
