@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2015 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -16,7 +16,7 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: nfsft.c 3896 2012-10-10 12:19:26Z tovo $ */
+/* $Id$ */
 
 /**
  * \file nfsft.c
@@ -39,7 +39,6 @@
 #endif
 
 /* Include NFFT3 utilities header. */
-#include "nfft3util.h"
 
 /* Include NFFT3 library header. */
 #include "nfft3.h"
@@ -264,7 +263,7 @@ void nfsft_init_advanced(nfsft_plan* plan, int N, int M,
                          unsigned int flags)
 {
   /* Call nfsft_init_guru with the flags and default NFFT cut-off. */
-  nfsft_init_guru(plan, N, M, flags, PRE_PHI_HUT | PRE_PSI | FFTW_INIT |
+  nfsft_init_guru(plan, N, M, flags, PRE_PHI_HUT | PRE_PSI | FFTW_INIT | NFFT_OMP_BLOCKWISE_ADJOINT |
                          FFT_OUT_OF_PLACE, NFSFT_DEFAULT_NFFT_CUTOFF);
 }
 
@@ -658,14 +657,16 @@ void nfsft_trafo_direct(nfsft_plan *plan)
   if (plan->flags & NFSFT_NORMALIZED)
   {
     /* Traverse Fourier coefficients array. */
+#ifdef _OPENMP
     #pragma omp parallel for default(shared) private(k,n)
+#endif
     for (k = 0; k <= plan->N; k++)
     {
       for (n = -k; n <= k; n++)
       {
         /* Multiply with normalization weight. */
         plan->f_hat_intern[NFSFT_INDEX(k,n,plan)] *=
-          sqrt((2*k+1)/(4.0*PI));
+          sqrt((2*k+1)/(4.0*KPI));
       }
     }
   }
@@ -690,13 +691,15 @@ void nfsft_trafo_direct(nfsft_plan *plan)
      *   = \sum_{n=-N}^N \sum_{k=|n|}^N a_k^n P_k^{|n|}(cos theta_m)
      *     e^{i n phi_m}.
      */
+#ifdef _OPENMP
     #pragma omp parallel for default(shared) private(m,stheta,sphi,f_m,n,a,n_abs,alpha,gamma,it2,it1,k,temp)
+#endif
     for (m = 0; m < plan->M_total; m++)
     {
       /* Scale angle theta from [0,1/2] to [0,pi] and apply cosine. */
-      stheta = cos(2.0*PI*plan->x[2*m+1]);
+      stheta = cos(2.0*KPI*plan->x[2*m+1]);
       /* Scale angle phi from [-1/2,1/2] to [-pi,pi]. */
-      sphi = 2.0*PI*plan->x[2*m];
+      sphi = 2.0*KPI*plan->x[2*m];
 
       /* Initialize result for current node. */
       f_m = 0.0;
@@ -812,9 +815,9 @@ void nfsft_adjoint_direct(nfsft_plan *plan)
         for (m = 0; m < plan->M_total; m++)
         {
           /* Scale angle theta from [0,1/2] to [0,pi] and apply cosine. */
-          stheta = cos(2.0*PI*plan->x[2*m+1]);
+          stheta = cos(2.0*KPI*plan->x[2*m+1]);
           /* Scale angle phi from [-1/2,1/2] to [-pi,pi]. */
-          sphi = 2.0*PI*plan->x[2*m];
+          sphi = 2.0*KPI*plan->x[2*m];
 
           /* Transposed Clenshaw algorithm */
 
@@ -845,9 +848,9 @@ void nfsft_adjoint_direct(nfsft_plan *plan)
     for (m = 0; m < plan->M_total; m++)
     {
       /* Scale angle theta from [0,1/2] to [0,pi] and apply cosine. */
-      stheta = cos(2.0*PI*plan->x[2*m+1]);
+      stheta = cos(2.0*KPI*plan->x[2*m+1]);
       /* Scale angle phi from [-1/2,1/2] to [-pi,pi]. */
-      sphi = 2.0*PI*plan->x[2*m];
+      sphi = 2.0*KPI*plan->x[2*m];
 
       /* Traverse all orders n. */
       for (n = -plan->N; n <= plan->N; n++)
@@ -892,14 +895,16 @@ void nfsft_adjoint_direct(nfsft_plan *plan)
   if (plan->flags & NFSFT_NORMALIZED)
   {
     /* Traverse Fourier coefficients array. */
+#ifdef _OPENMP
     #pragma omp parallel for default(shared) private(k,n)
+#endif
     for (k = 0; k <= plan->N; k++)
     {
       for (n = -k; n <= k; n++)
       {
         /* Multiply with normalization weight. */
         plan->f_hat[NFSFT_INDEX(k,n,plan)] *=
-          sqrt((2*k+1)/(4.0*PI));
+          sqrt((2*k+1)/(4.0*KPI));
       }
     }
   }
@@ -984,14 +989,16 @@ void nfsft_trafo(nfsft_plan *plan)
     if (plan->flags & NFSFT_NORMALIZED)
     {
       /* Traverse Fourier coefficients array. */
+#ifdef _OPENMP
       #pragma omp parallel for default(shared) private(k,n)
+#endif
       for (k = 0; k <= plan->N; k++)
       {
         for (n = -k; n <= k; n++)
         {
           /* Multiply with normalization weight. */
           plan->f_hat_intern[NFSFT_INDEX(k,n,plan)] *=
-            sqrt((2*k+1)/(4.0*PI));
+            sqrt((2*k+1)/(4.0*KPI));
         }
       }
     }
@@ -1228,14 +1235,16 @@ void nfsft_adjoint(nfsft_plan *plan)
       //fprintf(stderr,"nfsft_adjoint: Normalizing\n");
       //fflush(stderr);
       /* Traverse Fourier coefficients array. */
+#ifdef _OPENMP
       #pragma omp parallel for default(shared) private(k,n)
+#endif
       for (k = 0; k <= plan->N; k++)
       {
         for (n = -k; n <= k; n++)
         {
           /* Multiply with normalization weight. */
           plan->f_hat[NFSFT_INDEX(k,n,plan)] *=
-            sqrt((2*k+1)/(4.0*PI));
+            sqrt((2*k+1)/(4.0*KPI));
         }
       }
     }
@@ -1262,7 +1271,7 @@ void nfsft_precompute_x(nfsft_plan *plan)
   plan->plan_nfft.x = plan->x;
 
   /* Precompute. */
-  if (plan->plan_nfft.nfft_flags & PRE_ONE_PSI)
+  if (plan->plan_nfft.flags & PRE_ONE_PSI)
     nfft_precompute_one_psi(&plan->plan_nfft);
 }
 /* \} */
