@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2015 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -16,7 +16,7 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: nnfft.c 3896 2012-10-10 12:19:26Z tovo $ */
+/* $Id$ */
 
 #include "config.h"
 
@@ -27,7 +27,6 @@
 #ifdef HAVE_COMPLEX_H
 #include <complex.h>
 #endif
-#include "nfft3util.h"
 #include "nfft3.h"
 #include "infft.h"
 
@@ -37,10 +36,10 @@
 #define MACRO_nndft_init_result_adjoint memset(f_hat,0,ths->N_total*sizeof(double _Complex));
 #define MACRO_nndft_init_result_transposed MACRO_nndft_init_result_adjoint
 
-#define MACRO_nndft_sign_trafo      (-2.0*PI)
-#define MACRO_nndft_sign_conjugated (+2.0*PI)
-#define MACRO_nndft_sign_adjoint    (+2.0*PI)
-#define MACRO_nndft_sign_transposed (-2.0*PI)
+#define MACRO_nndft_sign_trafo      (-2.0*KPI)
+#define MACRO_nndft_sign_conjugated (+2.0*KPI)
+#define MACRO_nndft_sign_adjoint    (+2.0*KPI)
+#define MACRO_nndft_sign_transposed (-2.0*KPI)
 
 #define MACRO_nndft_compute_trafo (*fj) += (*f_hat_k)*cexp(+ _Complex_I*omega);
 
@@ -131,7 +130,7 @@ static void nnfft_uo(nnfft_plan *ths,int j,int *up,int *op,int act_dim)
                                 ths->psi[(ths->K+1)*t2+y_u[t2]+1]*            \
                                 (y[t2]-y_u[t2]))
 #define MACRO_with_PRE_PSI     ths->psi[(j*ths->d+t2)*(2*ths->m+2)+lj[t2]]
-#define MACRO_without_PRE_PSI  PHI(-ths->v[j*ths->d+t2]+                      \
+#define MACRO_without_PRE_PSI  PHI(ths->n[t2], -ths->v[j*ths->d+t2]+                      \
                                ((double)l[t2])/ths->N1[t2], t2)
 
 #define MACRO_init_uo_l_lj_t {                                                \
@@ -283,7 +282,7 @@ static inline void nnfft_D (nnfft_plan *ths){
 	  tmp = 1.0;
 	  /* multiply with N1, because x was modified */
 	  for(t=0; t<ths->d; t++)
-	      tmp*= 1.0 /((PHI_HUT(ths->x[ths->d*j + t]*((double)ths->N[t]),t)) );
+	      tmp*= 1.0 /((PHI_HUT(ths->n[t], ths->x[ths->d*j + t]*((double)ths->N[t]),t)) );
 	  ths->f[j] *= tmp;
       }
   }
@@ -303,6 +302,7 @@ void nnfft_trafo(nnfft_plan *ths)
     }
   }
 
+
   /* allows for external swaps of ths->f */
   ths->direct_plan->f = ths->f;
 
@@ -315,6 +315,7 @@ void nnfft_trafo(nnfft_plan *ths)
   }
 
   nnfft_D(ths);
+
 } /* nnfft_trafo */
 
 void nnfft_adjoint(nnfft_plan *ths)
@@ -357,7 +358,7 @@ void nnfft_precompute_phi_hut(nnfft_plan *ths)
     {
       tmp = 1.0;
       for(t=0; t<ths->d; t++)
-        tmp*= 1.0 /(PHI_HUT(ths->x[ths->d*j + t]*((double)ths->N[t]),t));
+        tmp*= 1.0 /(PHI_HUT(ths->n[t],ths->x[ths->d*j + t]*((double)ths->N[t]),t));
       ths->c_phi_inv[j]=tmp;
     }
 } /* nnfft_phi_hut */
@@ -378,7 +379,7 @@ void nnfft_precompute_lin_psi(nnfft_plan *ths)
       step=((double)(ths->m+1))/(ths->K*ths->N1[t]);
       for(j=0;j<=ths->K;j++)
         {
-          ths->psi[(ths->K+1)*t + j] = PHI(j*step,t);
+          ths->psi[(ths->K+1)*t + j] = PHI(ths->n[t],j*step,t);
         } /* for(j) */
     } /* for(t) */
 }
@@ -398,7 +399,7 @@ void nnfft_precompute_psi(nnfft_plan *ths)
 
         for(l=u, lj=0; l <= o; l++, lj++)
           ths->psi[(j*ths->d+t)*(2*ths->m+2)+lj]=
-            (PHI((-ths->v[j*ths->d+t]+((double)l)/((double)ths->N1[t])),t));
+            (PHI(ths->n[t],(-ths->v[j*ths->d+t]+((double)l)/((double)ths->N1[t])),t));
       } /* for(j) */
 
   for(j=0;j<ths->M_total;j++) {
@@ -479,6 +480,19 @@ void nnfft_precompute_full_psi(nnfft_plan *ths)
     } /* for(j) */
 }
 
+void nnfft_precompute_one_psi(nnfft_plan *ths)
+{
+  if(ths->nnfft_flags & PRE_PSI)
+    nnfft_precompute_psi(ths);
+  if(ths->nnfft_flags & PRE_FULL_PSI)
+    nnfft_precompute_full_psi(ths);
+  if(ths->nnfft_flags & PRE_LIN_PSI)
+    nnfft_precompute_lin_psi(ths);
+  /** precompute phi_hut, the entries of the matrix D */
+  if(ths->nnfft_flags & PRE_PHI_HUT)
+	  nnfft_precompute_phi_hut(ths);
+}
+
 static void nnfft_init_help(nnfft_plan *ths, int m2, unsigned nfft_flags, unsigned fftw_flags)
 {
   int t;                                /**< index over all dimensions       */
@@ -517,22 +531,28 @@ static void nnfft_init_help(nnfft_plan *ths, int m2, unsigned nfft_flags, unsign
 
   if(ths->nnfft_flags & MALLOC_X)
     ths->x = (double*)nfft_malloc(ths->d*ths->M_total*sizeof(double));
-  if(ths->nnfft_flags & MALLOC_F)
+  if(ths->nnfft_flags & MALLOC_F){
     ths->f=(double _Complex*)nfft_malloc(ths->M_total*sizeof(double _Complex));
-
+  }
   if(ths->nnfft_flags & MALLOC_V)
     ths->v = (double*)nfft_malloc(ths->d*ths->N_total*sizeof(double));
   if(ths->nnfft_flags & MALLOC_F_HAT)
     ths->f_hat = (double _Complex*)nfft_malloc(ths->N_total*sizeof(double _Complex));
 
+  //BUGFIX SUSE 2
+  /** precompute phi_hut, the entries of the matrix D */
+//  if(ths->nnfft_flags & PRE_PHI_HUT)
+//	  nnfft_precompute_phi_hut(ths);
+
   if(ths->nnfft_flags & PRE_LIN_PSI)
   {
     ths->K=(1U<< 10)*(ths->m+1);
     ths->psi = (double*) nfft_malloc((ths->K+1)*ths->d*sizeof(double));
   }
 
-  if(ths->nnfft_flags & PRE_PSI)
+  if(ths->nnfft_flags & PRE_PSI){
     ths->psi = (double*)nfft_malloc(ths->N_total*ths->d*(2*ths->m+2)*sizeof(double));
+  }
 
   if(ths->nnfft_flags & PRE_FULL_PSI)
   {
@@ -544,13 +564,11 @@ static void nnfft_init_help(nnfft_plan *ths, int m2, unsigned nfft_flags, unsign
       ths->psi_index_f = (int*) nfft_malloc(ths->N_total*sizeof(int));
       ths->psi_index_g = (int*) nfft_malloc(ths->N_total*lprod*sizeof(int));
   }
-
   ths->direct_plan = (nfft_plan*)nfft_malloc(sizeof(nfft_plan));
-
   nfft_init_guru(ths->direct_plan, ths->d, ths->aN1, ths->M_total, N2, m2,
 		 nfft_flags, fftw_flags);
-
   ths->direct_plan->x = ths->x;
+
   ths->direct_plan->f = ths->f;
   ths->F = ths->direct_plan->f_hat;
 
@@ -572,7 +590,7 @@ void nnfft_init_guru(nnfft_plan *ths, int d, int N_total, int M_total, int *N, i
   ths->m= m;
   ths->nnfft_flags= nnfft_flags;
   fftw_flags= FFTW_ESTIMATE| FFTW_DESTROY_INPUT;
-  nfft_flags= PRE_PHI_HUT| MALLOC_F_HAT| FFTW_INIT| FFT_OUT_OF_PLACE;
+  nfft_flags= PRE_PHI_HUT| MALLOC_F_HAT| FFTW_INIT| FFT_OUT_OF_PLACE| NFFT_OMP_BLOCKWISE_ADJOINT;
 
   if(ths->nnfft_flags & PRE_PSI)
     nfft_flags = nfft_flags | PRE_PSI;
@@ -599,44 +617,46 @@ void nnfft_init(nnfft_plan *ths, int d, int N_total, int M_total, int *N)
 
   unsigned nfft_flags;
   unsigned fftw_flags;
-
   ths->d = d;
   ths->M_total = M_total;
   ths->N_total = N_total;
-
   /* m should be greater to get the same accuracy as the nfft */
 /* Was soll dieser Ausdruck machen? Es handelt sich um eine Ganzzahl!
 
   WINDOW_HELP_ESTIMATE_m;
 */
+  //BUGFIX SUSE 1
+ths->m=WINDOW_HELP_ESTIMATE_m;
+
 
   ths->N = (int*) nfft_malloc(ths->d*sizeof(int));
   ths->N1 = (int*) nfft_malloc(ths->d*sizeof(int));
 
   for(t=0; t<d; t++) {
     ths->N[t] = N[t];
-
     /* the standard oversampling factor in the nnfft is 1.5 */
     ths->N1[t] = ceil(1.5*ths->N[t]);
-    
+
     /* N1 should be even */
     if(ths->N1[t]%2 != 0)
       ths->N1[t] = ths->N1[t] +1;
   }
 
   ths->nnfft_flags=PRE_PSI| PRE_PHI_HUT| MALLOC_X| MALLOC_V| MALLOC_F_HAT| MALLOC_F;
-  nfft_flags= PRE_PSI| PRE_PHI_HUT| MALLOC_F_HAT| FFTW_INIT| FFT_OUT_OF_PLACE;
+  nfft_flags= PRE_PSI| PRE_PHI_HUT| MALLOC_F_HAT| FFTW_INIT| FFT_OUT_OF_PLACE| NFFT_OMP_BLOCKWISE_ADJOINT;
 
   fftw_flags= FFTW_ESTIMATE| FFTW_DESTROY_INPUT;
-
   nnfft_init_help(ths,ths->m,nfft_flags,fftw_flags);
 }
 
+void nnfft_init_1d(nnfft_plan *ths,int N1, int M_total)
+{
+  nnfft_init(ths,1,N1,M_total,&N1);
+}
+
 void nnfft_finalize(nnfft_plan *ths)
 {
-
   nfft_finalize(ths->direct_plan);
-
   nfft_free(ths->direct_plan);
 
   nfft_free(ths->aN1);
