@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2015 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -16,7 +16,7 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: nsfft.c 3896 2012-10-10 12:19:26Z tovo $ */
+/* $Id$ */
 #include "config.h"
 
 #include <stdio.h>
@@ -26,7 +26,6 @@
 #ifdef HAVE_COMPLEX_H
 #include <complex.h>
 #endif
-#include "nfft3util.h"
 #include "nfft3.h"
 #include "infft.h"
 
@@ -55,7 +54,7 @@ static void short_nfft_trafo_2d(nfft_plan* ths, nfft_plan* plan_1d)
       for(j=0;j<ths->M_total;j++)
 	{
 	  omega = ((double)(k0 - ths->N[0]/2)) * ths->x[ths->d * j + 0];
-          ths->f[j] += plan_1d->f[j] * cexp( - I*2*PI*omega);
+          ths->f[j] += plan_1d->f[j] * cexp( - I*2*KPI*omega);
 	}
     }
 }
@@ -73,7 +72,7 @@ static void short_nfft_adjoint_2d(nfft_plan* ths, nfft_plan* plan_1d)
       for(j=0;j<ths->M_total;j++)
 	{
 	  omega = ((double)(k0 - ths->N[0]/2)) * ths->x[ths->d * j + 0];
-          plan_1d->f[j] = ths->f[j] * cexp( + _Complex_I*2*PI*omega);
+          plan_1d->f[j] = ths->f[j] * cexp( + _Complex_I*2*KPI*omega);
 	}
 
       plan_1d->f_hat = ths->f_hat + k0*ths->N[1];
@@ -107,7 +106,7 @@ static void short_nfft_trafo_3d_1(nfft_plan* ths, nfft_plan* plan_1d)
 	  {
 	    omega = ((double)(k0 - ths->N[0]/2)) * ths->x[ths->d * j + 0]
 	      +     ((double)(k1 - ths->N[1]/2)) * ths->x[ths->d * j + 1];
-            ths->f[j] += plan_1d->f[j] * cexp( - I*2*PI*omega);
+            ths->f[j] += plan_1d->f[j] * cexp( - I*2*KPI*omega);
 	  }
       }
 }
@@ -127,7 +126,7 @@ static void short_nfft_adjoint_3d_1(nfft_plan* ths, nfft_plan* plan_1d)
 	  {
 	    omega = ((double)(k0 - ths->N[0]/2)) * ths->x[ths->d * j + 0]
 	      +     ((double)(k1 - ths->N[1]/2)) * ths->x[ths->d * j + 1];
-            plan_1d->f[j] = ths->f[j] * cexp( + _Complex_I*2*PI*omega);
+            plan_1d->f[j] = ths->f[j] * cexp( + _Complex_I*2*KPI*omega);
 	  }
 
 	plan_1d->f_hat = ths->f_hat + (k0*ths->N[1]+k1)*ths->N[2];
@@ -160,7 +159,7 @@ static void short_nfft_trafo_3d_2(nfft_plan* ths, nfft_plan* plan_2d)
       for(j=0;j<ths->M_total;j++)
 	{
 	  omega = ((double)(k0 - ths->N[0]/2)) * ths->x[ths->d * j + 0];
-	  ths->f[j] += plan_2d->f[j] * cexp( - I*2*PI*omega);
+	  ths->f[j] += plan_2d->f[j] * cexp( - I*2*KPI*omega);
 	}
     }
 }
@@ -181,7 +180,7 @@ static void short_nfft_adjoint_3d_2(nfft_plan* ths, nfft_plan* plan_2d)
       for(j=0;j<ths->M_total;j++)
 	{
 	  omega = ((double)(k0 - ths->N[0]/2)) * ths->x[ths->d * j + 0];
-	  plan_2d->f[j] = ths->f[j] * cexp( + _Complex_I*2*PI*omega);
+	  plan_2d->f[j] = ths->f[j] * cexp( + _Complex_I*2*KPI*omega);
 	}
 
       plan_2d->f_hat = ths->f_hat + k0*ths->N[1]*ths->N[2];
@@ -226,8 +225,8 @@ static int index_sparse_to_full_direct_2d(int J, int k)
 	    o=j%4;                      /* kind of specific block            */
 	    a=X(exp2i)(r);
 	    b=X(exp2i)(J-r);
-	    l=NFFT_MAX(a,b);                 /* long dimension of block           */
-	    s=NFFT_MIN(a,b);                 /* short dimension of block          */
+	    l=MAX(a,b);                 /* long dimension of block           */
+	    s=MIN(a,b);                 /* short dimension of block          */
 	    m1=i/l;
 	    m2=i%l;
 
@@ -779,7 +778,7 @@ static void nsdft_trafo_2d(nsfft_plan *ths)
 	  omega =
 	    ((double)(k0 - N/2)) * ths->act_nfft_plan->x[2 * j + 0] +
 	    ((double)(k1 - N/2)) * ths->act_nfft_plan->x[2 * j + 1];
-          ths->f[j] += ths->f_hat[k_S] * cexp( - I*2*PI*omega);
+          ths->f[j] += ths->f_hat[k_S] * cexp( - I*2*KPI*omega);
 	}
     }
 } /* void nsdft_trafo_2d */
@@ -807,7 +806,7 @@ static void nsdft_trafo_3d(nsfft_plan *ths)
 	    ((double)(k0 - N/2)) * ths->act_nfft_plan->x[3 * j + 0] +
 	    ((double)(k1 - N/2)) * ths->act_nfft_plan->x[3 * j + 1] +
 	    ((double)(k2 - N/2)) * ths->act_nfft_plan->x[3 * j + 2];
-          ths->f[j] += ths->f_hat[k_S] * cexp( - I*2*PI*omega);
+          ths->f[j] += ths->f_hat[k_S] * cexp( - I*2*KPI*omega);
 	}
     }
 } /* void nsdft_trafo_3d */
@@ -839,7 +838,7 @@ static void nsdft_adjoint_2d(nsfft_plan *ths)
 	  omega =
 	    ((double)(k0 - N/2)) * ths->act_nfft_plan->x[2 * j + 0] +
 	    ((double)(k1 - N/2)) * ths->act_nfft_plan->x[2 * j + 1];
-          ths->f_hat[k_S] += ths->f[j] * cexp( + _Complex_I*2*PI*omega);
+          ths->f_hat[k_S] += ths->f[j] * cexp( + _Complex_I*2*KPI*omega);
 	}
     }
 } /* void nsdft_adjoint_2d */
@@ -867,7 +866,7 @@ static void nsdft_adjoint_3d(nsfft_plan *ths)
 	    ((double)(k0 - N/2)) * ths->act_nfft_plan->x[3 * j + 0] +
 	    ((double)(k1 - N/2)) * ths->act_nfft_plan->x[3 * j + 1] +
 	    ((double)(k2 - N/2)) * ths->act_nfft_plan->x[3 * j + 2];
-          ths->f_hat[k_S] += ths->f[j] * cexp( + _Complex_I*2*PI*omega);
+          ths->f_hat[k_S] += ths->f[j] * cexp( + _Complex_I*2*KPI*omega);
 	}
     }
 } /* void nsdft_adjoint_3d */
@@ -901,20 +900,20 @@ static void nsfft_trafo_2d(nsfft_plan *ths)
 
   for(rr=0;rr<=(J+1)/2;rr++)
     {
-      r=NFFT_MIN(rr,J-rr);
+      r=MIN(rr,J-rr);
       ths->act_nfft_plan->my_fftw_plan1 = ths->set_fftw_plan1[r];
       ths->act_nfft_plan->N[0]=X(exp2i)(r); ths->act_nfft_plan->n[0]=ths->sigma*ths->act_nfft_plan->N[0];
       ths->act_nfft_plan->N[1]=X(exp2i)(J-r); ths->act_nfft_plan->n[1]=ths->sigma*ths->act_nfft_plan->N[1];
 
       /*printf("%d x %d\n",ths->act_nfft_plan->N[0],ths->act_nfft_plan->N[1]);*/
 
-      temp=-3.0*PI*X(exp2i)(J-rr);
+      temp=-3.0*KPI*X(exp2i)(J-rr);
 
       /* right */
       ths->act_nfft_plan->f_hat=ths->f_hat+(4*rr+0)*X(exp2i)(J);
 
       if(r<rr)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -925,7 +924,7 @@ static void nsfft_trafo_2d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if(r<rr)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       for (j=0; j<M; j++)
         ths->f[j] +=  ths->act_nfft_plan->f[j] *
@@ -935,7 +934,7 @@ static void nsfft_trafo_2d(nsfft_plan *ths)
       ths->act_nfft_plan->f_hat=ths->f_hat+(4*rr+1)*X(exp2i)(J);
 
       if((r==rr)&&(J-rr!=rr))
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -946,7 +945,7 @@ static void nsfft_trafo_2d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if((r==rr)&&(J-rr!=rr))
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -956,7 +955,7 @@ static void nsfft_trafo_2d(nsfft_plan *ths)
       ths->act_nfft_plan->f_hat=ths->f_hat+(4*rr+2)*X(exp2i)(J);
 
       if(r<rr)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -967,7 +966,7 @@ static void nsfft_trafo_2d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if(r<rr)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -977,7 +976,7 @@ static void nsfft_trafo_2d(nsfft_plan *ths)
       ths->act_nfft_plan->f_hat=ths->f_hat+(4*rr+3)*X(exp2i)(J);
 
       if((r==rr)&&(J-rr!=rr))
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -988,7 +987,7 @@ static void nsfft_trafo_2d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if((r==rr)&&(J-rr!=rr))
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -1017,14 +1016,14 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
 
   for(rr=0;rr<=(J+1)/2;rr++)
     {
-      r=NFFT_MIN(rr,J-rr);
+      r=MIN(rr,J-rr);
       ths->act_nfft_plan->my_fftw_plan2 = ths->set_fftw_plan2[r];
       ths->act_nfft_plan->N[0]=X(exp2i)(r); ths->act_nfft_plan->n[0]=ths->sigma*ths->act_nfft_plan->N[0];
       ths->act_nfft_plan->N[1]=X(exp2i)(J-r); ths->act_nfft_plan->n[1]=ths->sigma*ths->act_nfft_plan->N[1];
 
       /*printf("%d x %d\n",ths->act_nfft_plan->N[0],ths->act_nfft_plan->N[1]);*/
 
-      temp=-3.0*PI*X(exp2i)(J-rr);
+      temp=-3.0*KPI*X(exp2i)(J-rr);
 
       /* right */
       ths->act_nfft_plan->f_hat=ths->f_hat+(4*rr+0)*X(exp2i)(J);
@@ -1034,7 +1033,7 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
                                   cexp( - I*temp*ths->act_nfft_plan->x[2*j+1]);
 
       if(r<rr)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1045,7 +1044,7 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if(r<rr)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       /* top */
       ths->act_nfft_plan->f_hat=ths->f_hat+(4*rr+1)*X(exp2i)(J);
@@ -1055,7 +1054,7 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
                                   cexp( - I*temp*ths->act_nfft_plan->x[2*j+0]);
 
       if((r==rr)&&(J-rr!=rr))
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1066,7 +1065,7 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if((r==rr)&&(J-rr!=rr))
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       /* left */
       ths->act_nfft_plan->f_hat=ths->f_hat+(4*rr+2)*X(exp2i)(J);
@@ -1076,7 +1075,7 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
                                   cexp( + _Complex_I*temp*ths->act_nfft_plan->x[2*j+1]);
 
       if(r<rr)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1087,7 +1086,7 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if(r<rr)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       /* bottom */
       ths->act_nfft_plan->f_hat=ths->f_hat+(4*rr+3)*X(exp2i)(J);
@@ -1097,7 +1096,7 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
                                   cexp( + _Complex_I*temp*ths->act_nfft_plan->x[2*j+0]);
 
       if((r==rr)&&(J-rr!=rr))
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1108,7 +1107,7 @@ static void nsfft_adjoint_2d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if((r==rr)&&(J-rr!=rr))
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_transposed);
+	RSWAP(ths->act_nfft_plan->x,ths->x_transposed);
     } /* for(rr) */
 } /* void nsfft_adjoint_2d */
 
@@ -1140,7 +1139,7 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 
       N_B_r=a*b*b;
 
-      r=NFFT_MIN(rr,J-rr);
+      r=MIN(rr,J-rr);
       ths->act_nfft_plan->my_fftw_plan1 = ths->set_fftw_plan1[rr];
 
       ths->act_nfft_plan->N[0]=X(exp2i)(r);
@@ -1160,15 +1159,15 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 
       /* only for right - rear - top */
       if((J==0)||((J==1)&&(rr==1)))
-	temp=-2.0*PI;
+	temp=-2.0*KPI;
       else
-	temp=-3.0*PI*X(exp2i)(J-rr);
+	temp=-3.0*KPI*X(exp2i)(J-rr);
 
       /* right */
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*0;
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_120);
+	RSWAP(ths->act_nfft_plan->x,ths->x_120);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1182,7 +1181,7 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_120);
+	RSWAP(ths->act_nfft_plan->x,ths->x_120);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -1192,9 +1191,9 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*1;
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_021);
+	RSWAP(ths->act_nfft_plan->x,ths->x_021);
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_102);
+	RSWAP(ths->act_nfft_plan->x,ths->x_102);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1208,9 +1207,9 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_021);
+	RSWAP(ths->act_nfft_plan->x,ths->x_021);
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_102);
+	RSWAP(ths->act_nfft_plan->x,ths->x_102);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -1220,7 +1219,7 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*2;
 
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_201);
+	RSWAP(ths->act_nfft_plan->x,ths->x_201);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1234,7 +1233,7 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_201);
+	RSWAP(ths->act_nfft_plan->x,ths->x_201);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -1242,15 +1241,15 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 
       /* only for left - front - bottom */
       if((J==0)||((J==1)&&(rr==1)))
-	temp=-4.0*PI;
+	temp=-4.0*KPI;
       else
-	temp=-3.0*PI*X(exp2i)(J-rr);
+	temp=-3.0*KPI*X(exp2i)(J-rr);
 
       /* left */
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*3;
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_120);
+	RSWAP(ths->act_nfft_plan->x,ths->x_120);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1264,7 +1263,7 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_120);
+	RSWAP(ths->act_nfft_plan->x,ths->x_120);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -1274,9 +1273,9 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*4;
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_021);
+	RSWAP(ths->act_nfft_plan->x,ths->x_021);
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_102);
+	RSWAP(ths->act_nfft_plan->x,ths->x_102);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1290,9 +1289,9 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_021);
+	RSWAP(ths->act_nfft_plan->x,ths->x_021);
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_102);
+	RSWAP(ths->act_nfft_plan->x,ths->x_102);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -1302,7 +1301,7 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*5;
 
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_201);
+	RSWAP(ths->act_nfft_plan->x,ths->x_201);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1316,7 +1315,7 @@ static void nsfft_trafo_3d(nsfft_plan *ths)
 	nfft_trafo(ths->act_nfft_plan);
 
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_201);
+	RSWAP(ths->act_nfft_plan->x,ths->x_201);
 
       for (j=0; j<M; j++)
         ths->f[j] += ths->act_nfft_plan->f[j] *
@@ -1354,7 +1353,7 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
 
       N_B_r=a*b*b;
 
-      r=NFFT_MIN(rr,J-rr);
+      r=MIN(rr,J-rr);
       ths->act_nfft_plan->my_fftw_plan1 = ths->set_fftw_plan1[rr];
       ths->act_nfft_plan->my_fftw_plan2 = ths->set_fftw_plan2[rr];
 
@@ -1375,9 +1374,9 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
 
       /* only for right - rear - top */
       if((J==0)||((J==1)&&(rr==1)))
-	temp=-2.0*PI;
+	temp=-2.0*KPI;
       else
-	temp=-3.0*PI*X(exp2i)(J-rr);
+	temp=-3.0*KPI*X(exp2i)(J-rr);
 
       /* right */
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*0;
@@ -1387,7 +1386,7 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
                                   cexp( - I*temp*ths->act_nfft_plan->x[3*j+0]);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_120);
+	RSWAP(ths->act_nfft_plan->x,ths->x_120);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1401,7 +1400,7 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_120);
+	RSWAP(ths->act_nfft_plan->x,ths->x_120);
 
       /* rear */
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*1;
@@ -1411,9 +1410,9 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
                                   cexp( - I*temp*ths->act_nfft_plan->x[3*j+1]);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_021);
+	RSWAP(ths->act_nfft_plan->x,ths->x_021);
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_102);
+	RSWAP(ths->act_nfft_plan->x,ths->x_102);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1427,9 +1426,9 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_021);
+	RSWAP(ths->act_nfft_plan->x,ths->x_021);
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_102);
+	RSWAP(ths->act_nfft_plan->x,ths->x_102);
 
       /* top */
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*2;
@@ -1439,7 +1438,7 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
                                   cexp( - I*temp*ths->act_nfft_plan->x[3*j+2]);
 
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_201);
+	RSWAP(ths->act_nfft_plan->x,ths->x_201);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1453,13 +1452,13 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_201);
+	RSWAP(ths->act_nfft_plan->x,ths->x_201);
 
       /* only for left - front - bottom */
       if((J==0)||((J==1)&&(rr==1)))
-	temp=-4.0*PI;
+	temp=-4.0*KPI;
       else
-	temp=-3.0*PI*X(exp2i)(J-rr);
+	temp=-3.0*KPI*X(exp2i)(J-rr);
 
       /* left */
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*3;
@@ -1469,7 +1468,7 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
                                   cexp( + _Complex_I*temp*ths->act_nfft_plan->x[3*j+0]);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_120);
+	RSWAP(ths->act_nfft_plan->x,ths->x_120);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1483,7 +1482,7 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_120);
+	RSWAP(ths->act_nfft_plan->x,ths->x_120);
 
       /* front */
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*4;
@@ -1493,9 +1492,9 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
                                   cexp( + _Complex_I*temp*ths->act_nfft_plan->x[3*j+1]);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_021);
+	RSWAP(ths->act_nfft_plan->x,ths->x_021);
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_102);
+	RSWAP(ths->act_nfft_plan->x,ths->x_102);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1509,9 +1508,9 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if(a>b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_021);
+	RSWAP(ths->act_nfft_plan->x,ths->x_021);
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_102);
+	RSWAP(ths->act_nfft_plan->x,ths->x_102);
 
       /* bottom */
       ths->act_nfft_plan->f_hat=ths->f_hat + sum_N_B_less_r + N_B_r*5;
@@ -1521,7 +1520,7 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
                                   cexp( + _Complex_I*temp*ths->act_nfft_plan->x[3*j+2]);
 
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_201);
+	RSWAP(ths->act_nfft_plan->x,ths->x_201);
 
       if(ths->act_nfft_plan->N[0]<=ths->act_nfft_plan->m)
 	if(ths->act_nfft_plan->N[1]<=ths->act_nfft_plan->m)
@@ -1535,7 +1534,7 @@ static void nsfft_adjoint_3d(nsfft_plan *ths)
 	nfft_adjoint(ths->act_nfft_plan);
 
       if(a<b)
-	NFFT_SWAP_double(ths->act_nfft_plan->x,ths->x_201);
+	RSWAP(ths->act_nfft_plan->x,ths->x_201);
 
       sum_N_B_less_r+=6*N_B_r;
     } /* for(rr) */
@@ -1593,7 +1592,7 @@ static void nsfft_init_2d(nsfft_plan *ths, int J, int M, int m, unsigned snfft_f
 
   nfft_init_guru(ths->act_nfft_plan,2,N,M,n,m, FG_PSI| MALLOC_X| MALLOC_F| FFTW_INIT, FFTW_MEASURE);
 
-  if(ths->act_nfft_plan->nfft_flags & PRE_ONE_PSI)
+  if(ths->act_nfft_plan->flags & PRE_ONE_PSI)
     nfft_precompute_one_psi(ths->act_nfft_plan);
 
   ths->set_fftw_plan1[0]=ths->act_nfft_plan->my_fftw_plan1;
@@ -1618,7 +1617,7 @@ static void nsfft_init_2d(nsfft_plan *ths, int J, int M, int m, unsigned snfft_f
       N[0]=X(exp2i)(J-r); n[0]=ths->sigma*N[0]; /* ==N[1] of the 2 dimensional plan */
 
       nfft_init_guru(&(ths->set_nfft_plan_1d[r]),1,N,M,n,m, MALLOC_X| MALLOC_F| FFTW_INIT, FFTW_MEASURE);
-      ths->set_nfft_plan_1d[r].nfft_flags = ths->set_nfft_plan_1d[r].nfft_flags | FG_PSI;
+      ths->set_nfft_plan_1d[r].flags = ths->set_nfft_plan_1d[r].flags | FG_PSI;
       ths->set_nfft_plan_1d[r].K=ths->act_nfft_plan->K;
       ths->set_nfft_plan_1d[r].psi=ths->act_nfft_plan->psi;
     }
@@ -1630,7 +1629,7 @@ static void nsfft_init_2d(nsfft_plan *ths, int J, int M, int m, unsigned snfft_f
   nfft_init_guru(ths->center_nfft_plan,2,N,M,n, m, MALLOC_F| FFTW_INIT,
                      FFTW_MEASURE);
   ths->center_nfft_plan->x= ths->act_nfft_plan->x;
-  ths->center_nfft_plan->nfft_flags = ths->center_nfft_plan->nfft_flags|
+  ths->center_nfft_plan->flags = ths->center_nfft_plan->flags|
                                       FG_PSI;
   ths->center_nfft_plan->K=ths->act_nfft_plan->K;
   ths->center_nfft_plan->psi=ths->act_nfft_plan->psi;
@@ -1684,7 +1683,7 @@ static void nsfft_init_3d(nsfft_plan *ths, int J, int M, int m, unsigned snfft_f
 
   nfft_init_guru(ths->act_nfft_plan,3,N,M,n,m, FG_PSI| MALLOC_X| MALLOC_F, FFTW_MEASURE);
 
-  if(ths->act_nfft_plan->nfft_flags & PRE_ONE_PSI)
+  if(ths->act_nfft_plan->flags & PRE_ONE_PSI)
     nfft_precompute_one_psi(ths->act_nfft_plan);
 
   /* malloc g1, g2 for maximal size */
@@ -1706,7 +1705,7 @@ static void nsfft_init_3d(nsfft_plan *ths, int J, int M, int m, unsigned snfft_f
       a=X(exp2i)(J-rr);
       b=X(exp2i)(rr);
 
-      r=NFFT_MIN(rr,J-rr);
+      r=MIN(rr,J-rr);
 
       n[0]=ths->sigma*X(exp2i)(r);
       if(a<b)
@@ -1732,11 +1731,11 @@ static void nsfft_init_3d(nsfft_plan *ths, int J, int M, int m, unsigned snfft_f
       if(N[0]>m)
 	{
 	  nfft_init_guru(&(ths->set_nfft_plan_1d[r]),1,N,M,n,m, MALLOC_X| MALLOC_F| FFTW_INIT, FFTW_MEASURE);
-	  ths->set_nfft_plan_1d[r].nfft_flags = ths->set_nfft_plan_1d[r].nfft_flags | FG_PSI;
+	  ths->set_nfft_plan_1d[r].flags = ths->set_nfft_plan_1d[r].flags | FG_PSI;
 	  ths->set_nfft_plan_1d[r].K=ths->act_nfft_plan->K;
 	  ths->set_nfft_plan_1d[r].psi=ths->act_nfft_plan->psi;
 	  nfft_init_guru(&(ths->set_nfft_plan_2d[r]),2,N,M,n,m, MALLOC_X| MALLOC_F| FFTW_INIT, FFTW_MEASURE);
-	  ths->set_nfft_plan_2d[r].nfft_flags = ths->set_nfft_plan_2d[r].nfft_flags | FG_PSI;
+	  ths->set_nfft_plan_2d[r].flags = ths->set_nfft_plan_2d[r].flags | FG_PSI;
 	  ths->set_nfft_plan_2d[r].K=ths->act_nfft_plan->K;
 	  ths->set_nfft_plan_2d[r].psi=ths->act_nfft_plan->psi;
 	}
@@ -1750,7 +1749,7 @@ static void nsfft_init_3d(nsfft_plan *ths, int J, int M, int m, unsigned snfft_f
   nfft_init_guru(ths->center_nfft_plan,3,N,M,n, m, MALLOC_F| FFTW_INIT,
                      FFTW_MEASURE);
   ths->center_nfft_plan->x= ths->act_nfft_plan->x;
-  ths->center_nfft_plan->nfft_flags = ths->center_nfft_plan->nfft_flags|
+  ths->center_nfft_plan->flags = ths->center_nfft_plan->flags|
                                       FG_PSI;
   ths->center_nfft_plan->K=ths->act_nfft_plan->K;
   ths->center_nfft_plan->psi=ths->act_nfft_plan->psi;
@@ -1799,14 +1798,14 @@ static void nsfft_finalize_2d(nsfft_plan *ths)
     nfft_free(ths->index_sparse_to_full);
 
   /* center plan */
-  ths->center_nfft_plan->nfft_flags = ths->center_nfft_plan->nfft_flags ^ FG_PSI;
+  ths->center_nfft_plan->flags = ths->center_nfft_plan->flags ^ FG_PSI;
   nfft_finalize(ths->center_nfft_plan);
 
   /* the 1d nffts */
   for(r=0;r<=X(log2i)(ths->act_nfft_plan->m);r++)
     {
-      ths->set_nfft_plan_1d[r].nfft_flags =
-        ths->set_nfft_plan_1d[r].nfft_flags ^ FG_PSI;
+      ths->set_nfft_plan_1d[r].flags =
+        ths->set_nfft_plan_1d[r].flags ^ FG_PSI;
       nfft_finalize(&(ths->set_nfft_plan_1d[r]));
     }
 
@@ -1842,7 +1841,7 @@ static void nsfft_finalize_3d(nsfft_plan *ths)
     nfft_free(ths->index_sparse_to_full);
 
   /* center plan */
-  ths->center_nfft_plan->nfft_flags = ths->center_nfft_plan->nfft_flags ^ FG_PSI;
+  ths->center_nfft_plan->flags = ths->center_nfft_plan->flags ^ FG_PSI;
   nfft_finalize(ths->center_nfft_plan);
 
   /* the 1d and 2d nffts */
@@ -1850,9 +1849,9 @@ static void nsfft_finalize_3d(nsfft_plan *ths)
     {
       if(X(exp2i)(ths->J-r)>ths->act_nfft_plan->m)
 	{
-	  ths->set_nfft_plan_2d[r].nfft_flags = ths->set_nfft_plan_2d[r].nfft_flags ^ FG_PSI;
+	  ths->set_nfft_plan_2d[r].flags = ths->set_nfft_plan_2d[r].flags ^ FG_PSI;
 	  nfft_finalize(&(ths->set_nfft_plan_2d[r]));
-	  ths->set_nfft_plan_1d[r].nfft_flags = ths->set_nfft_plan_1d[r].nfft_flags ^ FG_PSI;
+	  ths->set_nfft_plan_1d[r].flags = ths->set_nfft_plan_1d[r].flags ^ FG_PSI;
 	  nfft_finalize(&(ths->set_nfft_plan_1d[r]));
 	}
     }
