@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2012 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2015 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -20,7 +20,7 @@
 
 #include "infft.h"
 
-R X(sinc)(const R x)
+R Y(sinc)(const R x)
 {
   /* Based on sinc function from Boost C++ library. */
   const R b =  EPSILON;
